from __future__ import annotations

from typing import Final

import requests
from requests import Response
from requests.adapters import HTTPAdapter

from ..core.entry import SignInEntry
from ..base.request import cf_detected, NetworkState
# Removed reseed functionality
from ..schema.nexusphp import AttendanceHR
from ..utils import net_utils
from ..utils.value_handler import size


class MainClass(AttendanceHR):
    URL: Final = 'https://piggo.me/'
    USER_CLASSES: Final = {
        'downloaded': [size(2, 'TiB'), size(6, 'TiB')],
        'share_ratio': [4, 6],
        'points': [400000, 1000000],
        'days': [280, 700]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#info_block > tbody > tr > td > table > tbody > tr > td:nth-child(2) > span',
                    }
                }
            },
            'details': {
                'points': {
                    'regex': (r'(做种积分).*?([\d,.]+)', 2)
                }
            }
        })
        return selector

    def request(self,
                entry: SignInEntry,
                method: str,
                url: str,
                **kwargs,
                ) -> Response | None:
        if not self.session:
            self.session = requests.Session()
            if entry_headers := entry.get('headers'):
                self.session.headers.update(entry_headers)
            if entry_cookie := entry.get('cookie'):
                self.session.cookies.update(net_utils.cookie_str_to_dict(entry_cookie))
            self.session.mount('http://', HTTPAdapter(max_retries=2))
            self.session.mount('https://', HTTPAdapter(max_retries=2))
        try:
            response: Response = self.session.request(method, url, timeout=60, **kwargs)
            if cf_detected(response):
                entry.fail_with_prefix('Detected CloudFlare DDoS-GUARD')
            elif response is not None and response.status_code != 200:
                if not (response.status_code == 500 and 'messages.php' in response.url):
                    entry.fail_with_prefix(f'response.status_code={response.status_code}')
            entry['session_cookie'] = (' '.join(list(map(lambda x: f'{x[0]}={x[1]};', self.session.cookies.items()))))
            return response
        except Exception as e:
            entry.fail_with_prefix(NetworkState.NETWORK_ERROR.value.format(url=url, error=e))
        return None



from typing import Final

from ..schema.nexusphp import Attendance


class MainClass(Attendance):
    URL: Final = 'https://ultrahd.net/'
    USER_CLASSES: Final = {
        'downloaded': [2199023255552, 8796093022208],
        'share_ratio': [5, 6.5],
        'days': [175, 280]
    }



from abc import ABC

from .private_torrent import PrivateTorrent
from ..utils.value_handler import handle_join_date


class Unit3D(PrivateTorrent, ABC):
    @property
    def details_selector(self) -> dict:
        selector = {
            'user_id': '/users/(.*?)"',
            'detail_sources': {
                'default': {
                    'link': '/users/{}',
                    'elements': {
                        'bar': '#main-content > div.ratio-bar > div > ul',
                        'data_table': '.gradient'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': (r'(上传|Upload).+?([\d.]+.?[ZEPTGMK]?iB)', 2)
                },
                'downloaded': {
                    'regex': (r'(下载|Download).+?([\d.]+.?[ZEPTGMK]?iB)', 2)
                },
                'share_ratio': {
                    'regex': (r'(分享率|Ratio).+?([\d.]+)', 2)
                },
                'points': {
                    'regex': (r'(魔力|BON).+?(\d[\d,. ]*)', 2),
                    'handle': self.handle_points
                },
                'join_date': {
                    'regex': (r'(注册日期|Registration date) (.*?\d{4})', 2),
                    'handle': handle_join_date
                },
                'seeding': {
                    'regex': (r'(做种|Seeding).+?(\d+)', 2)
                },
                'leeching': {
                    'regex': (r'(吸血|Leeching).+?(\d+)', 2)
                },
                'hr': {
                    'regex': (r'(警告|Warnings).+?(\d+)', 2)
                }
            }
        }
        return selector

    def handle_points(self, value: str) -> str:
        return value.replace(' ', '')



import datetime
import re
from abc import ABC
from urllib.parse import urljoin

from bs4 import BeautifulSoup

from .private_torrent import PrivateTorrent
from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
from ..utils import net_utils
from ..utils.value_handler import handle_infinite


class Gazelle(PrivateTorrent, ABC):

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_gazelle_message(entry, config)

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': r'user\.php\?id=(\d+)',
            'detail_sources': {
                'default': {
                    'link': '/user.php?id={}',
                    'elements': {'table': '#content > div > div.sidebar > div.box.box_info.box_userinfo_stats > ul'}
                }
            },
            'details': {
                'uploaded': {
                    'regex': (r'(Upload|上传量).+?([\d.]+ ?[ZEPTGMK]?i?B)', 2)
                },
                'downloaded': {
                    'regex': (r'(Download|下载量).+?([\d.]+ ?[ZEPTGMK]?i?B)', 2)
                },
                'share_ratio': {
                    'regex': (r'(Ratio|分享率).*?(∞|[\d,.]+)', 2),
                    'handle': handle_infinite
                },
                'points': {
                    'regex': (r'(Gold|积分|Bonus|Credits|Nips).*?([\d,.]+)', 2)
                },
                'join_date': {
                    'regex': ('(Joined|加入时间).*?(.*?)(ago|前|Last seen)', 2),
                    'handle': self.handle_join_date
                },
                'seeding': {
                    'regex': r'[Ss]eeding.+?([\d,]+)'
                },
                'leeching': {
                    'regex': r'[Ll]eeching.+?([\d,]+)'
                },
                'hr': None
            }
        }

    def get_gazelle_message(self, entry: SignInEntry, config: dict,
                            message_body_selector: str = 'div[id*="message"]') -> None:
        message_url = urljoin(entry['url'], '/inbox.php')
        message_box_response = self.request(entry, 'get', message_url)
        network_state = check_network_state(entry, message_url, message_box_response)
        if network_state != NetworkState.SUCCEED:
            return
        unread_elements = BeautifulSoup(net_utils.decode(message_box_response, "html.parser")).select("tr.unreadpm > td > strong > a")
        failed = False
        for unread_element in unread_elements:
            title = unread_element.text
            href = unread_element.get('href')
            message_url = urljoin(message_url, href)
            message_response = self.request(entry, 'get', message_url)
            network_state = check_network_state(entry, message_url, message_response)
            message_body = 'Can not read message body!'
            if network_state != NetworkState.SUCCEED:
                failed = True
            else:
                body_element = BeautifulSoup(
                    net_utils.decode(message_response, "html.parser")).select_one(message_body_selector)
                if body_element:
                    message_body = body_element.text.strip()
            entry['messages'] = entry['messages'] + (
                '\nTitle: {}\nLink: {}\n{}'.format(title, message_url, message_body))
        if failed:
            entry.fail_with_prefix('Can not read message body!')

    def handle_join_date(self, value: str) -> datetime.date:
        year_regex = '(\\d+) (年|years?)'
        month_regex = '(\\d+) (月|months?)'
        week_regex = '(\\d+) (周|weeks?)'
        year = 0
        month = 0
        week = 0
        if year_match := re.search(year_regex, value):
            year = int(year_match.group(1))
        if month_match := re.search(month_regex, value):
            month = int(month_match.group(1))
        if week_match := re.search(week_regex, value):
            week = int(week_match.group(1))
        return (datetime.datetime.now() - datetime.timedelta(days=year * 365 + month * 31 + week * 7)).date()



from typing import Final

from ..core.entry import SignInEntry
# Removed reseed functionality
from ..base.sign_in import check_final_state, SignState
from ..base.work import Work
from ..schema.nexusphp import AttendanceHR
from ..utils import net_utils


class MainClass(AttendanceHR):
    URL: Final = 'https://bitporn.eu/'
    USER_CLASSES: Final = {
        'downloaded': [805306368000, 3298534883328],
        'share_ratio': [3, 4.55],
        'points': [50000, 500000],
        'days': [224, 700]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/attendance.php',
                method=self.sign_in_by_get,
                succeed_regex=[
                    'You have already attended .* days, Continuous.* days, this time you will get .* bonus.',
                    'Attend got: \\d+'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'link': '/userdetails.php?id={}',
                    'elements': {
                        'bar': '.tracker-stats',
                        'table': '#outer table.main:last-child'
                    }
                }
            },
            'details': {
                'points': {
                    'regex': r'Seed points([\d,.]+)'
                },
                'seeding': {
                    'regex': r'Active.*?(\d+)'
                },
                'leeching': {
                    'regex': r'Active.*?\d+.*?(\d+)'
                },
                'hr': {
                    'regex': r'H&R.*?(\d+)'
                }
            }
        })
        return selector



from __future__ import annotations

import re
from abc import ABC, abstractmethod
from enum import Enum

from loguru import logger
from requests import Response

from ..core.entry import SignInEntry
from .request import NetworkState, check_network_state
from .work import Work


class SignState(Enum):
    NO_SIGN_IN = 'No sign in'
    SUCCEED = 'Succeed'
    WRONG_ANSWER = 'Wrong answer'
    SIGN_IN_FAILED = 'Sign in failed, {}'
    UNKNOWN = 'Unknown, url: {}'


class NetworkErrorReason(Enum):
    DDOS_PROTECTION_BY_CLOUDFLARE = 'DDoS protection by .+?Cloudflare'
    SERVER_LOAD_TOO_HIGH = r'<h3 align=center>(服务器负载过|伺服器負載過)高，正在重(试|試)，(请|請)稍(后|後)\.\.\.</h3>'
    CONNECTION_TIMED_OUT = r'<h2 class="text-gray-600 leading-1\.3 text-3xl font-light">Connection timed out</h2>'
    THE_WEB_SERVER_REPORTED_A_BAD_GATEWAY_ERROR = r'<p>The web server reported a bad gateway error\.</p>'
    WEB_SERVER_IS_DOWN = '站点关闭维护中，请稍后再访问...谢谢|站點關閉維護中，請稍後再訪問...謝謝|Web server is down'
    INCORRECT_CSRF_TOKEN = 'Incorrect CSRF token'


def check_state(entry: SignInEntry,
                work: Work,
                response: Response | None,
                content: str | None,
                ) -> bool:
    if entry.failed:
        return False
    if not work.assert_state:
        return True
    check_method, state = work.assert_state
    return check_method(entry, work, response, content) == state


def check_sign_in_state(entry: SignInEntry,
                        work: Work,
                        response: Response | None,
                        content: str | None,
                        ) -> NetworkState | SignState:
    if (network_state := check_network_state(entry, work, response, content=content,
                                             check_content=True)) != NetworkState.SUCCEED:
        return network_state
    if not (succeed_regex := work.succeed_regex):
        entry['result'] = SignState.SUCCEED.value + entry.get('extra_msg', '')
        return SignState.SUCCEED
    for regex in succeed_regex:
        if isinstance(regex, str):
            regex = (regex, 0)
        regex, group_index = regex
        if succeed_msg := re.search(regex, content):
            entry['result'] = re.sub('<.*?>|&shy;|&nbsp;', '', succeed_msg.group(group_index)) + entry[
                'result'] + entry.get('extra_msg', '')
            return SignState.SUCCEED
    if (fail_regex := work.fail_regex) and re.search(fail_regex, content):
        return SignState.WRONG_ANSWER
    for reason in NetworkErrorReason:
        if re.search(reason.value, content):
            entry.fail_with_prefix(
                NetworkState.NETWORK_ERROR.value.format(url=work.url, error=reason.name.title()))
            return NetworkState.NETWORK_ERROR
    if (assert_state := work.assert_state) and assert_state[1] != SignState.NO_SIGN_IN:
        logger.warning(f'no sign in, regex: {succeed_regex}, content: {content}')
    return SignState.NO_SIGN_IN


def check_final_state(entry: SignInEntry,
                      work: Work,
                      response: Response,
                      content: str,
                      ) -> SignState:
    if (sign_in_state := check_sign_in_state(entry, work, response, content)) == SignState.NO_SIGN_IN:
        entry.fail_with_prefix(SignState.SIGN_IN_FAILED.value.format('no sign in'))
        return SignState.SIGN_IN_FAILED
    return sign_in_state


class SignIn(ABC):
    @classmethod
    @abstractmethod
    def sign_in_build_schema(cls) -> dict:
        pass

    @classmethod
    @abstractmethod
    def sign_in_build_entry(cls, entry: SignInEntry, config: dict) -> None:
        pass

    @abstractmethod
    def sign_in(self, entry: SignInEntry, config: dict) -> None:
        pass



import re
from typing import Final

from ..core.entry import SignInEntry
from ..base.request import NetworkState, check_network_state
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.ocelot import Ocelot
from ..utils.net_utils import get_module_name


class MainClass(Ocelot):
    URL: Final = 'https://filelist.io/'
    USER_CLASSES: Final = {
        'downloaded': [45079976738816],
        'share_ratio': [5],
        'days': [1460]
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'},
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/login.php',
                method=self.sign_in_by_get,
                assert_state=(check_network_state, NetworkState.SUCCEED),
            ),
            Work(
                url='/takelogin.php',
                method=self.sign_in_by_login,
                succeed_regex=['Hello, <a .+?</a>'],
                response_urls=['/my.php'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
            )
        ]

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            'validator': re.search("(?<='validator' value=').*(?=')", last_content).group(),
            'username': login['username'],
            'password': login['password'],
            'unlock': 1
        }



from __future__ import annotations

import re
from typing import Final
from urllib.parse import urljoin

from flexget.entry import Entry
from flexget.utils.soup import get_soup

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
# Removed reseed functionality
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.unit3d import Unit3D
from ..utils import net_utils
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_join_date, handle_infinite


class MainClass(Unit3D, Reseed):
    URL: Final = 'https://pt.hdpost.top'
    USER_CLASSES: Final = {
        'uploaded': [109951162777600],
        'days': [365]
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'cookie': {'type': 'string'},
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    @classmethod
    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/login',
                method=self.sign_in_by_get,
                assert_state=(check_network_state, NetworkState.SUCCEED),
            ),
            Work(
                url='/login',
                method=self.sign_in_by_login,
                assert_state=(check_network_state, NetworkState.SUCCEED),
                response_urls=['', '/pages/1'],
            )
        ]

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=[('https://pt.hdpost.top/users/(.*?)/', 1)],
                assert_state=(check_final_state, SignState.SUCCEED),
                use_last_content=True,
                is_base_content=True,
                response_urls=['', '/pages/1'],
            )
        ]

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        login_page = get_soup(last_content)
        hidden_input = login_page.select_one('form > input:nth-last-child(2)')
        name = hidden_input.attrs['name']
        value = hidden_input.attrs['value']
        return {
            '_token': re.search(r'(?<=name="_token" value=").+?(?=")', last_content).group(),
            'username': login['username'],
            'password': login['password'],
            'remember': 'on',
            '_captcha': re.search(r'(?<=name="_captcha" value=").+?(?=")', last_content).group(),
            '_username': '',
            name: value,
        }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'user_id': '/users/(.*?)/',
            'detail_sources': {
                'default': {
                    'do_not_strip': True,
                    'elements': {
                        'bar': 'ul.top-nav__ratio-bar',
                        'header': '.profile__registration',
                        'data_table': 'aside'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': '上传.+?([\\d.]+ [ZEPTGMK]?iB)',
                             'handle': self.remove_symbol
                },
                'downloaded': {
                    'regex': '下载.+?([\\d.]+ [ZEPTGMK]?iB)',
                    'handle': self.remove_symbol
                },
                'share_ratio': {
                    'regex': '分享率.+?([\\d.]+)',
                    'handle': handle_infinite
                },
                'points': {
                    'regex': '魔力.+?(\\d[\\d,. ]*)',
                    'handle': self.handle_points
                },
                'join_date': {
                    'regex': r'注册日期.*?(\d{4}-\d{2}-\d{2})',
                    'handle': handle_join_date
                },
                'seeding': {
                    'regex': '做种.+?(\\d+)'
                },
                'leeching': {
                    'regex': '吸血.+?(\\d+)'
                },
                'hr': {
                    'regex': 'H&amp;R.+?(\\d+)'
                }
            }
        })
        return selector

    @classmethod
    def remove_symbol(self, value: str):
        return value.replace('\xa0', '')


from typing import Final
from urllib.parse import urljoin

from flexget.utils.soup import get_soup

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.gazelle import Gazelle
from ..utils import net_utils


class MainClass(Gazelle):
    URL: Final = 'https://www.empornium.sx/'
    USER_CLASSES: Final = {
        'uploaded': [107374182400],
        'share_ratio': [1.05],
        'days': [56]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['<h1 class="hidden">Empornium</h1>'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': 'table.userinfo_stats',
                        'table': '#content > div > div.sidebar > div:nth-child(4) > ul',
                        'community': '#content > div > div.sidebar > div:nth-child(10) > ul'
                    }
                }
            }
        })
        return selector

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_gazelle_message(entry, config)

    def get_gazelle_message(self, entry: SignInEntry, config: dict,
                            message_body_selector: str = 'div[id*="message"]') -> None:
        message_url = urljoin(entry['url'], '/user/inbox/received')
        message_box_response = self.request(entry, 'get', message_url)
        network_state = check_network_state(entry, message_url, message_box_response)
        if network_state != NetworkState.SUCCEED:
            return
        unread_elements = get_soup(net_utils.decode(message_box_response)).select("tr.unreadpm > td > strong > a")
        failed = False
        for unread_element in unread_elements:
            title = unread_element.text
            href = unread_element.get('href')
            message_url = urljoin(message_url, href)
            message_response = self.request(entry, 'get', message_url)
            network_state = check_network_state(entry, message_url, message_response)
            message_body = 'Can not read message body!'
            if network_state != NetworkState.SUCCEED:
                failed = True
            else:
                body_element = get_soup(
                    net_utils.decode(message_response)).select_one(message_body_selector)
                if body_element:
                    message_body = body_element.text.strip()
            entry['messages'] = entry['messages'] + (
                '\nTitle: {}\nLink: {}\n{}'.format(title, message_url, message_body))
        if failed:
            entry.fail_with_prefix('Can not read message body!')



from typing import Final

from ..schema.avistaz import AvistaZ


class MainClass(AvistaZ):
    URL: Final = 'https://cinemaz.to/'
    SUCCEED_REGEX: Final = '<h1 class="title"><i class="fa fa-home"></i> CinemaZ Beta 3.0</h1>'



from typing import Final

from ..core.entry import SignInEntry
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.private_torrent import PrivateTorrent
from ..utils.value_handler import handle_join_date, handle_infinite


class MainClass(PrivateTorrent):
    # IPTorrents in list of https://flexget.com/URLRewriters
    URL: Final = 'https://iptorrents.com/download.php/8/none.torrent'
    USER_CLASSES: Final = {
        'uploaded': [53687091200],
        'share_ratio': [1.05],
        'days': [28]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['Log out'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
                response_urls=['/t']
            )
        ]

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': '"/u/(\\d+)"',
            'detail_sources': {
                'default': {
                    'link': '/user.php?u={}',
                    'elements': {
                        'bar': 'div.stats',
                        'table': 'table#body table.t1'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': 'Uploaded.+?([\\d.]+ [ZEPTGMK]?B)'
                },
                'downloaded': {
                    'regex': 'Downloaded.+?([\\d.]+ [ZEPTGMK]?B)'
                },
                'share_ratio': {
                    'regex': r'Ratio (-|[\d,.]+)',
                    'handle': handle_infinite
                },
                'points': {
                    'regex': r'Bonus Points\s+([\d,.]+)'
                },
                'join_date': {
                    'regex': 'Join date\\s*?(\\d{4}-\\d{2}-\\d{2})',
                    'handle': handle_join_date
                },
                'seeding': {
                    'regex': 'Seeding([\\d,]+)'
                },
                'leeching': {
                    'regex': 'Leeching([\\d,]+)'
                },
                'hr': None
            }
        }



from typing import Final
from urllib.parse import urljoin

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
# Removed reseed functionality
from ..base.sign_in import Work
from ..schema.nexusphp import VisitHR
from ..utils import google_auth
from ..utils.net_utils import get_module_name


class MainClass(VisitHR):
    URL: Final = 'https://ourbits.club/'
    USER_CLASSES: Final = {
        'downloaded': [2199023255552, 8796093022208],
        'share_ratio': [4, 5.5],
        'days': [175, 364]
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'cookie': {'type': 'string'},
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'},
                            'secret_key': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            '2fa_code': login.get('secret_key') and google_auth.calc(login['secret_key']) or '',
            'trackerssl': 'yes',
            'username': login['username'],
            'password': login['password'],
            'returnto': 'attendance.php'
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/takelogin.php',
                method=self.sign_in_by_login,
                assert_state=(check_network_state, NetworkState.SUCCEED),
                response_urls=['/attendance.php']
            )
        ]

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        entry['extra_msg'] = f'未签到: {urljoin(self.URL, "/attendance.php")}'
        workflow = super().sign_in_build_workflow(entry, config)
        workflow[0].use_last_content = True
        return workflow



from typing import Final

# Removed reseed functionality
from ..schema.nexusphp import Visit
from ..utils import net_utils


class MainClass(Visit):
    URL: Final = 'https://pt.keepfrds.com/'
    USER_CLASSES: Final = {
        'downloaded': [1099511627776, 4398046511104],
        'share_ratio': [3.5, 5],
        'points': [640000, 2560000],
        'days': [420, 1050]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#info_block > tbody > tr > td > table > tbody > tr > td:nth-child(1)'
                    }
                }
            }
        })
        return selector



from typing import Final

# Removed reseed functionality
from ..schema.nexusphp import Attendance
from ..utils.value_handler import size


class MainClass(Attendance):
    URL: Final = 'https://www.hdkyl.in/'
    USER_CLASSES: Final = {
        'downloaded': [size(750, 'GiB'), size(10, 'TiB')],
        'share_ratio': [6, 10],
        'points': [400000, 1000000],
        'days': [280, 700]
    }



from typing import Final

from ..core.entry import SignInEntry
from ..schema.nexusphp import Visit
from ..utils.value_handler import handle_infinite


class MainClass(Visit):
    URL: Final = 'https://www.zootracker.vip/'
    @property
    def SUCCEED_REGEX(self) -> str:
        return 'You are logged in as: .*?(?=<)'

    USER_CLASSES: Final = {
        'downloaded': [2147483648000, 1759218604442],
        'share_ratio': [4, 6],
        'days': [70, 1092]
    }

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        pass

    @property
    def details_selector(self) -> dict:
        return {
            'detail_sources': {
                'default': {
                    'link': '/account.php',
                    'elements': {
                        'bar': 'body > table:nth-child(2) > tbody > tr > td:nth-child(2) > table > tbody:nth-child(2) > tr > td > table > tbody:nth-child(2) > tr > td:nth-child(3) > table:nth-child(2) > tbody > tr > td > table:nth-child(2) > tbody > tr > td.bodymain2',
                        'table': '.f-border.comment'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': (r'(Uploaded).+?([\d.]+ ?[ZEPTGMK]?i?B)', 2)
                },
                'downloaded': {
                    'regex': (r'(Downloaded).+?([\d.]+ ?[ZEPTGMK]?i?B)', 2)
                },
                'share_ratio': {
                    'regex': (r'(Ratio).*?(---|∞|Inf\.|无限|無限|[\d,.]+)', 2),
                    'handle': handle_infinite
                },
                'points': None,
                'join_date': {
                    'regex': (r'(Joined).*?(\d{4}-\d{2}-\d{2})', 2),
                },
                'seeding': None,
                'leeching': None,
                'hr': None
            }
        }


from typing import Final

from ..core.entry import SignInEntry
from ..base.sign_in import check_final_state, SignState
from ..base.work import Work
from ..schema.unit3d import Unit3D
from ..utils import net_utils
from ..utils.value_handler import handle_join_date


class MainClass(Unit3D):
    URL: Final = 'https://aither.cc/'
    USER_CLASSES: Final = {
        'uploaded': [43980465111040],
        'days': [365]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['<title>Aither - Heaven</title>'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'user_id': '/users/(.*?)/',
            'detail_sources': {
                'default': {
                    'do_not_strip': True,
                    'elements': {
                        'bar': 'ul.top-nav__ratio-bar',
                        'registration_date': 'article > time',
                        'warnings': 'article > aside > section:nth-child(2)',
                        'data_table': 'article > aside > section:nth-child(4)'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': r'Upload.+?([\d.]+.?[ZEPTGMK]?iB)',
                    'handle': self.remove_symbol
                },
                'downloaded': {
                    'regex': r'Download.+?([\d.]+.?[ZEPTGMK]?iB)',
                    'handle': self.remove_symbol
                },
                'points': {
                    'regex': 'title="My Bonus Points".*?</i>.+?(\\d[\\d,. ]*)',
                },
                'share_ratio': {
                    'regex': 'title="Ratio".*?</i>.+?(\\d[\\d,. ]*)',
                },
                'join_date': {
                    'regex': r'Registration date.*?(\d{4}-\d{2}-\d{2})',
                    'handle': handle_join_date
                },
                'hr': {
                    'regex': r'Hit and Run Count.*?(\d+)'
                }
            }
        })
        return selector

    def remove_symbol(self, value: str):
        return value.replace('\xa0', '')



from typing import Final

from ..core.entry import SignInEntry
from ..schema.xbtit import XBTIT
from ..utils import net_utils
from ..utils.value_handler import handle_infinite


class MainClass(XBTIT):
    URL: Final = 'https://sportscult.org/'
    SUCCEED_REGEX: Final = 'Welcome back .*?</b>'
    USER_CLASSES: Final = {
        'uploaded': [268435456000],
        'share_ratio': [1.8]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'user_id': 'index.php\\?page=usercp&amp;uid=(\\d+)',
            'detail_sources': {
                'default': {
                    'link': '/index.php?page=usercp&uid={}',
                    'elements': {
                        'bar': '.b-content table.lista',
                        'table': '#bodyarea #mcol .block-content-r table.lista'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': '↑\\s*([\\d.]+ [ZEPTGMK]B)'
                },
                'downloaded': {
                    'regex': '↓\\s*([\\d.]+ [ZEPTGMK]B)'
                },
                'share_ratio': {
                    'regex': 'Ratio (---|[\\d.]+)',
                    'handle': handle_infinite

                },
                'points': {
                    'regex': 'Bonus (---|[\\d,.]+)',
                    'handle': handle_infinite
                },
                'join_date': {
                    'regex': r'Joined on:[^\d]+(.*?\d{4})',
                    'handle': self.handle_join_date
                },
                'seeding': None,
                'leeching': None,
                'hr': None
            }
        })
        return selector

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_XBTIT_message(entry, config,
                               MESSAGES_URL_REGEX='index.php\\?page=usercp&amp;uid=\\d+&amp;do=pm&amp;action=list')



from __future__ import annotations

import re
from typing import Final
from urllib.parse import urljoin

from requests import Response

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
# Removed reseed functionality
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.nexusphp import NexusPHP
from ..utils import net_utils


class MainClass(NexusPHP):
    URL: Final = 'https://pt.btschool.club/'
    USER_CLASSES: Final = {
        'downloaded': [1099511627776, 10995116277760],
        'share_ratio': [3.05, 4.55],
        'points': [600000, 1000000],
        'days': [280, 700]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/index.php?action=addbonus',
                method=self.sign_in_by_location,
                succeed_regex=['欢迎回来'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            ),
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'details': {
                'points': {
                    'regex': r'做种积分:.*?([\d.,]+)',
                }
            }
        })
        return selector

    def sign_in_by_location(self, entry: SignInEntry, config: dict, work: Work, last_content: str) -> Response | None:
        response = self.request(entry, 'get', work.url)
        reload__net_state = check_network_state(entry, work.url, response)
        if reload__net_state != NetworkState.SUCCEED:
            return None
        content = net_utils.decode(response)
        location_search = re.search('(?<=window\\.location=).*?(?=;)', content)
        if not location_search:
            return response
        location_url = re.sub('["+ ]', '', location_search.group(0))
        work.url = urljoin(MainClass.URL, location_url)
        return self.sign_in_by_get(entry, config, work)



import hashlib
from typing import Final

from ..core.entry import SignInEntry
from ..base.request import NetworkState, check_network_state
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.private_torrent import PrivateTorrent
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_infinite


class MainClass(PrivateTorrent):
    URL: Final = 'https://www.gay-torrents.net/'

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/login.php?do=login',
                method=self.sign_in_by_login,
                succeed_regex=[r'Thank you for logging in, .*?\.</p>'],
                assert_state=(check_network_state, NetworkState.SUCCEED),
                response_urls=['/login.php?do=login']
            )
        ]

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/latest/',
                method=self.sign_in_by_get,
                succeed_regex=['Log Out'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
                response_urls=['/latest/']
            )
        ]

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            'do': 'login',
            'vb_login_md5password': hashlib.md5(login['password'].encode()).hexdigest(),
            'vb_login_md5password_utf': hashlib.md5(login['password'].encode()).hexdigest(),
            's': '',
            'securitytoken': 'guest',
            'url': '/latest/',
            'vb_login_username': login['username'],
            'vb_login_password': login['password'],
            'cookieuser': 1
        }

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': r'<a href="member\.php\?([-\w]+?)">My Profile</a>',
            'detail_sources': {
                'default': {
                    'link': '/member.php?{}',
                    'elements': {
                        'table1': '#view-stats_mini > div > div > div',
                        'table2': '#view-stats_mini.subsection > div > div > div'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': r'Uploaded\s*([\d.]+ ([ZEPTGMK]i)?B)'
                },
                'downloaded': {
                    'regex': r'Downloaded\s*([\d.]+ ([ZEPTGMK]i)?B)'
                },
                'share_ratio': {
                    'regex': r'Ratio(∞|[\d,.]+)',
                    'handle': handle_infinite
                },
                'points': {
                    'regex': r'Juices([\d,.]+)'
                },
                'join_date': {
                    'regex': r'Join Date\s*?[\d:]+? (.+?)\s',
                },
                'seeding': None,
                'leeching': None,
                'hr': None
            }
        }



import re
from typing import Final

from ..core.entry import SignInEntry
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.private_torrent import PrivateTorrent
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_join_date, handle_infinite


class MainClass(PrivateTorrent):
    URL: Final = 'https://ssl.bootytape.com/'

    USER_CLASSES: Final = {
        'uploaded': [214748364800],
        'share_ratio': [1],
        'days': [31],
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/login.php',
                method=self.sign_in_by_login,
                succeed_regex=['logout'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
                response_urls=['/my.php']
            )
        ]

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            'take_login': 1,
            'username': login['username'],
            'password': login['password'],
        }

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': fr'{re.escape("Welcome, <a href=userdetails.php?id=")}(\d+)',
            'detail_sources': {
                'default': {
                    'link': '/userdetails.php?id={}',
                    'elements': {
                        'table': 'body > table.mainouter > tbody > tr:nth-child(2) > td > table:nth-child(12)',
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': r"""(?x)Uploaded
                                [\d.] +
                                \ 
                                [ZEPTGMKk] ?
                                B
                                \ 
                                \(
                                ([\d,] +)""",
                    'handle': self.handle_amount_of_data
                },
                'downloaded': {
                    'regex': r"""(?x)Downloaded
                                . *?
                                \(
                                ([\d,] +)""",
                    'handle': self.handle_amount_of_data
                },
                'share_ratio': {
                    'regex': r"""(?x)Share
                                \ 
                                ratio
                                (∞ | [\d,.] +)""",
                    'handle': handle_infinite
                },
                'points': {
                    'regex': r"""(?x)Seed
                                \ 
                                Bonus
                                ([\d,.] +)"""
                },
                'join_date': {
                    'regex': r"""(?x)Join
                                \s
                                date
                                (. +?)
                                \ """,
                    'handle': handle_join_date
                },
                'seeding': None,
                'leeching': None,
                'hr': None
            }
        }

    def handle_amount_of_data(self, value: str) -> str:
        return value + 'B'



from __future__ import annotations

import json
from io import BytesIO
from typing import Final
from urllib.parse import urljoin

from requests import Response

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
# Removed reseed functionalityPage
from ..base.sign_in import check_final_state, SignState, check_sign_in_state
from ..base.work import Work
from ..schema.nexusphp import NexusPHP
from ..utils import net_utils, baidu_ocr
from ..utils.net_utils import get_module_name

try:
    from PIL import Image
except ImportError:
    Image = None


class MainClass(NexusPHP):
    URL: Final = 'https://hdsky.me/'
    IMAGE_HASH_URL: Final = '/image_code_ajax.php'
    IMAGE_URL: Final = '/image.php?action=regimage&imagehash={}'
    TORRENT_PAGE_URL: Final = '/details.php?id={torrent_id}&hit=1'
    DOWNLOAD_URL_REGEX: Final = '/download\\.php\\?id=\\d+&passkey=.*?(?=")'
    USER_CLASSES: Final = {
        'downloaded': [8796093022208, 10995116277760],
        'share_ratio': [5, 5.5],
        'days': [315, 455]
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'cookie': {'type': 'string'},
                    'join_date': {'type': 'string', 'format': 'date'}
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['已签到', 'Showed Up'],
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True,
            ),
            Work(
                url='/showup.php',
                method=self.sign_in_by_post,
                succeed_regex=['已签到', 'Showed Up', 'success'],
                fail_regex='失败|error|failed',
                assert_state=(check_final_state, SignState.SUCCEED),
            ),
        ]

    def sign_in_by_post(self, entry: SignInEntry, config: dict, work: Work,
                        last_content: str | None = None) -> Response | None:
        """简单的POST签到，不使用验证码"""
        data = {
            'action': 'showup'
        }
        return self.request(entry, 'post', work.url, data=data)

    def sign_in_by_ocr(self, entry: SignInEntry, config: dict, work: Work, last_content: str) -> Response | None:
        # 获取验证码hash
        data = {
            'action': 'new'
        }
        image_hash_url = urljoin(entry['url'], self.IMAGE_HASH_URL)
        image_hash_response = self.request(entry, 'post', image_hash_url, data=data)
        image_hash_network_state = check_network_state(entry, image_hash_url, image_hash_response)
        if image_hash_network_state != NetworkState.SUCCEED:
            return None
        content = net_utils.decode(image_hash_response)
        if not content:
            entry.fail_with_prefix('Empty response content')
            return None

        try:
            response_data = json.loads(content)
            image_hash = response_data.get('code')
        except (json.JSONDecodeError, KeyError) as e:
            entry.fail_with_prefix(f'Invalid JSON response: {e}. Content: {content[:100]}...')
            return None

        if not image_hash:
            entry.fail_with_prefix('Cannot find: image_hash')
            return None
        image_url = urljoin(entry['url'], self.IMAGE_URL)
        img_url = image_url.format(image_hash)
        img_response = self.request(entry, 'get', img_url)
        img_network_state = check_network_state(entry, img_url, img_response)
        if img_network_state != NetworkState.SUCCEED:
            return None
        img = Image.open(BytesIO(img_response.content))
        code, img_byte_arr = baidu_ocr.get_ocr_code(img, entry, config)
        if code and len(code) == 6:
            # 提交签到
            data = {
                'action': 'showup',
                'imagehash': image_hash,
                'imagestring': code
            }
            return self.request(entry, 'post', work.url, data=data)
        return None

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'user_id': None,
            'detail_sources': {
                'default': {
                    'link': None,
                    'elements': {
                        'bar': '#info_block > tbody > tr > td > table > tbody > tr > td:nth-child(1) > span',
                        'table': None
                    }
                }
            },
            'details': {
                'join_date': None,
                'hr': None
            }
        })
        return selector



from __future__ import annotations

import itertools
import json
import re
from abc import ABC
from pathlib import Path
from time import sleep
from urllib.parse import urljoin

from bs4 import BeautifulSoup
from loguru import logger

from .private_torrent import PrivateTorrent
from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
from ..base.sign_in import SignState, check_final_state, check_sign_in_state
from ..base.work import Work
from ..utils import net_utils
from ..utils.value_handler import handle_infinite


class NexusPHP(PrivateTorrent, ABC):

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_nexusphp_messages(entry, config)

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': r'userdetails\.php\?id=(\d+)',
            'detail_sources': {
                'default': {
                    'link': '/userdetails.php?id={}',
                    'elements': {
                        'bar': '#info_block',
                        'table': '#outer table.main:last-child'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': (r'(上[传傳]量|Uploaded).+?([\d.]+ ?[ZEPTGMK]?i?B)', 2)
                },
                'downloaded': {
                    'regex': (r'(下[载載]量|Downloaded).+?([\d.]+ ?[ZEPTGMK]?i?B)', 2)
                },
                'share_ratio': {
                    'regex': (r'(分享率|Ratio).*?(---|∞|Inf\.|无限|無限|[\d,.]+)', 2),
                    'handle': handle_infinite
                },
                'points': {
                    'regex': (r'(魔力|Bonus|Bônus).*?([\d,.]+)', 2)
                },
                'join_date': {
                    'regex': (r'(加入日期|注册日期|Join.date|Data de Entrada).*?(\d{4}-\d{2}-\d{2})', 2),
                },
                'seeding': {
                    'regex': (r'(当前活动|當前活動|Torrents Ativos).*?(\d+)', 2)
                },
                'leeching': {
                    'regex': (r'(当前活动|當前活動|Torrents Ativos).*?\d+\D+(\d+)', 2)
                },
                'hr': {
                    'regex': r'H&R.*?(\d+)'
                }
            }
        }

    def get_nexusphp_messages(self, entry: SignInEntry, config: dict,
                              messages_url: str = '/messages.php?action=viewmailbox&box=1&unread=yes',
                              unread_elements_selector: str = 'td > img[alt*="Unread"]',
                              ignore_title: str | None = None) -> None:
        message_url = urljoin(entry['url'], messages_url)
        message_box_response = self.request(entry, 'get', message_url)
        message_box_network_state = check_network_state(entry, message_url, message_box_response)
        if message_box_network_state != NetworkState.SUCCEED:
            entry.fail_with_prefix(f'Can not read message box! url:{message_url}')
            return

        unread_elements = BeautifulSoup(net_utils.decode(message_box_response), "html.parser").select(
            unread_elements_selector)
        failed = False
        for unread_element in unread_elements:
            td = unread_element.parent.next_sibling.next_sibling
            title = td.text
            href = td.a.get('href')
            message_url = urljoin(message_url, href)
            message_response = self.request(entry, 'get', message_url)
            message_network_state = check_network_state(entry, message_url, message_response)
            if message_network_state != NetworkState.SUCCEED:
                message_body = 'Can not read message body!'
                failed = True
            else:
                if body_element := BeautifulSoup(net_utils.decode(message_response), "html.parser").select_one('td[colspan*="2"]'):
                    message_body = body_element.text.strip()
                else:
                    message_body = 'Can not find message body element!'
            if ignore_title and re.match(ignore_title, title):
                logger.info(f'\nIgnore Title: {title}\nLink: {message_url}\n{message_body}')
                continue
            entry['messages'] = entry['messages'] + f'\nTitle: {title}\nLink: {message_url}\n{message_body}'
        if failed:
            entry.fail_with_prefix('Can not read message body!')


class AttendanceHR(NexusPHP, ABC):
    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/attendance.php',
                method=self.sign_in_by_get,
                succeed_regex=[
                    '这是您的第.*?次签到，已连续签到.*?天，本次签到获得.*?魔力值。|這是您的第.*次簽到，已連續簽到.*?天，本次簽到獲得.*?魔力值。',
                    '[签簽]到已得\\d+',
                    '您今天已经签到过了，请勿重复刷新。|您今天已經簽到過了，請勿重複刷新。'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]


class Attendance(AttendanceHR, ABC):
    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'details': {
                'hr': None
            }
        })
        return selector


class BakatestHR(NexusPHP, ABC):
    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/bakatest.php',
                method=self.sign_in_by_get,
                succeed_regex=['今天已经签过到了\\(已连续.*天签到\\)'],
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True
            ),
            Work(
                url='/bakatest.php',
                method=self.sign_in_by_question,
                succeed_regex=['连续.*天签到,获得.*点魔力值|今天已经签过到了\\(已连续.*天签到\\)'],
                fail_regex='回答错误,失去 1 魔力值,这道题还会再考一次',
            )
        ]

    def sign_in_by_question(self, entry: SignInEntry, config: dict, work: Work, last_content: str = None) -> None:
        question_element = BeautifulSoup(last_content, "html.parser").select_one('input[name="questionid"]')
        if question_element:
            question_id = question_element.get('value')

            question_file = Path(__file__).parent.parent.joinpath(f'data/{entry["site_name"]}.json')

            if question_file.is_file():
                question_json = json.loads(question_file.read_text(encoding='utf-8'))
            else:
                question_json = {}

            local_answer = question_json.get(question_id)

            choice_elements = BeautifulSoup(last_content, "html.parser").select('input[name="choice[]"]')
            choices = []
            for choice_element in choice_elements:
                choices.append(choice_element.get('value', ''))

            if choice_elements[0].get('type') == 'radio':
                choice_range = 1
            else:
                choice_range = len(choices)

            answer_list = []

            for i in range(choice_range):
                for arr in itertools.combinations(choices, i + 1):
                    if list(arr) not in answer_list:
                        answer_list.append(list(arr))
            answer_list.reverse()
            if local_answer and len(local_answer) == len([i for i in local_answer if i in choices]) and len(
                    local_answer) <= choice_range:
                answer_list.insert(0, local_answer)
            times = 0
            for answer in answer_list:
                data = {'questionid': question_id, 'choice[]': answer, 'usercomment': '此刻心情:无', 'submit': '提交'}
                response = self.request(entry, 'post', work.url, data=data)
                state = check_sign_in_state(entry, work, response, net_utils.decode(response))
                if state == SignState.SUCCEED:
                    entry['result'] = f"{entry['result']} ( {times} attempts.)"
                    if question_json.get(question_id) != answer:
                        question_json[question_id] = answer
                        question_file.write_text(
                            json.dumps({int(x): question_json[x] for x in question_json.keys()}, indent=4),
                            encoding='utf-8')
                    logger.info(f"{entry['title']}, correct answer: {data}")
                    return
                times += 1
                sleep(3)
        entry.fail_with_prefix(SignState.SIGN_IN_FAILED.value.format('No answer.'))


class Bakatest(BakatestHR, ABC):
    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'details': {
                'hr': None
            }
        })
        return selector


class VisitHR(NexusPHP, ABC):
    @property
    def SUCCEED_REGEX(self) -> str:
        return '[欢歡]迎回[来來家]'

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=[self.SUCCEED_REGEX],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]


class Visit(VisitHR, ABC):
    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'details': {
                'hr': None
            }
        })
        return selector



from typing import Final

from ..core.entry import SignInEntry
# Removed reseed functionality
from ..base.sign_in import check_final_state, SignState, check_sign_in_state
from ..base.work import Work
from ..schema.nexusphp import NexusPHP
from ..utils import net_utils


class MainClass(NexusPHP):
    URL: Final = 'https://pterclub.com/'
    IGNORE_TITLE = '认领种子获得猫粮60000克'
    USER_CLASSES: Final = {
        'downloaded': [805306368000, 3298534883328],
        'share_ratio': [3.05, 4.55],
        'days': [210, 315]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['签到已得\\d+'],
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True
            ),
            Work(
                url='/attendance-ajax.php',
                method=self.sign_in_by_get,
                succeed_regex=[
                    '这是您的第 .* 次签到，已连续签到 .* 天。.*本次签到获得 .* 克猫粮。',
                    '签到已得\\d+',
                    '您今天已经签到过了，请勿重复刷新。'
                ],
                assert_state=(check_final_state, SignState.SUCCEED),
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'table': '#outer table.main'
                    }
                }
            },
            'details': {
                'points': {
                    'regex': '猫粮.*?([\\d,.]+)'
                }
            }
        })
        return selector

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_nexusphp_messages(entry, config, ignore_title=self.IGNORE_TITLE)



from typing import Final

from ..core.entry import SignInEntry
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.gazelle import Gazelle
from ..utils import net_utils


class MainClass(Gazelle):
    URL: Final = 'https://uhdbits.org/'
    USER_CLASSES: Final = {
        'downloaded': [322122547200],
        'share_ratio': [2.0],
        'days': [56]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['<h1 class="hidden">UHDBits</h1>'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': 'ul#userinfo_stats',
                        'table': 'div.sidebar > div:nth-child(2) > ul'
                    }
                },
                'extend': {
                    'link': '/ajax.php?action=community_stats&userid={}'
                }
            }
        })
        return selector



import json
import re
from io import BytesIO
from pathlib import Path
from time import sleep
from typing import Final
from urllib.parse import urljoin

import numpy as np
import requests
from PIL import Image
from flexget.utils.soup import get_soup
from loguru import logger

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
# Removed reseed functionality
from ..base.sign_in import check_final_state, SignState, check_sign_in_state
from ..base.work import Work
from ..schema.nexusphp import NexusPHP
from ..utils import net_utils


class MainClass(NexusPHP):
    URL: Final = 'https://tjupt.org/'
    BREAK_REGEX: Final = r'已断签.*?天，当前可补签天数为 <b>(\d+)</b> 天'
    IMG_SELECTOR: Final = 'table.captcha img'
    ANSWER_SELECTOR: Final = 'table.captcha form > table > tbody > tr:nth-child(2) > td'
    CONFIRM: Final = {'action': 'confirm'}
    CANCEL: Final = {'action': 'cancel'}
    IGNORE_TITLE = '您正在下载或做种的种子被删除'
    USER_CLASSES: Final = {
        'uploaded': [5368709120000, 53687091200000],
        'days': [336, 924]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        entry['extra_msg'] = f'未签到: {urljoin(self.URL, "/attendance.php")}'
        return [
            Work(
                # url='/attendance.php',
                url='/',
                method=self.sign_in_by_get,
                # succeed_regex=['今日已签到'],
                succeed_regex=['欢迎'],
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True
            ),
            Work(
                url='/attendance.php',
                method=self.sign_in_by_douban,
                succeed_regex=['这是您的首次签到，本次签到获得.*?个魔力值。',
                               '签到成功，这是您的第.*?次签到，已连续签到.*?天，本次签到获得.*?个魔力值。',
                               '重新签到成功，本次签到获得.*?个魔力值'],
                assert_state=(check_final_state, SignState.SUCCEED)
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#info_block > tbody > tr > td',
                    }
                }
            },
            'details': {
                'downloaded': None,
                'share_ratio': None,
                'seeding': {
                    'regex': '种子数合计.*?(\\d+)'
                },
                'leeching': {
                    'regex': '种子数合计.*?\\d+\\D+(\\d+)'
                },
                'hr': {
                    'regex': 'H&R.*?(\\d+)',
                    'handle': self.handle_hr
                }

            }
        })
        return selector

    def sign_in_by_douban(self, entry: SignInEntry, config: dict, work: Work, last_content: str):
        if break_match := re.search(self.BREAK_REGEX, last_content):
            if int(break_match.group(1)) > 0:
                params = self.CONFIRM
            else:
                params = self.CANCEL
            response = self.request(entry, 'get', work.url, params=params)
            network_state = check_network_state(entry, response.url, response)
            if not network_state == NetworkState.SUCCEED:
                return
            last_content = net_utils.decode(response)
        captcha_el = get_soup(last_content).select_one(self.IMG_SELECTOR)
        answer_el = get_soup(last_content).select_one(self.ANSWER_SELECTOR)
        value = list(map(lambda x: x['value'], answer_el.select('input')))
        answer = get_soup(str(answer_el).replace('<br/>', '|')).text.split('|')[0:-1]

        answers = list(zip(value, answer))

        captcha_img_url = urljoin(self.URL, captcha_el['src'])

        answer = self.get_answer(entry, config, captcha_img_url, answers)
        if not answer:
            return
        data = {
            'answer': answer,
            'submit': '提交'
        }
        return self.request(entry, 'post', work.url, data=data)

    def get_answer(self, entry, config, captcha_img_url, answers):
        question_file = Path(__file__).parent.parent.joinpath(f'data/{entry["site_name"]}.json')
        if question_file.is_file():
            question_json = json.loads(question_file.read_text(encoding='utf-8'))
        else:
            question_json = {}
        img_name = captcha_img_url.split('/')[-1]
        cache_question = question_json.get(img_name)
        if cache_question:
            cache_answer = cache_question.get('answer')
            for value, answer in answers:
                if cache_answer == answer:
                    return value
            entry.fail_with_prefix('cache error!')
            return None

        captcha_img_response = self.request(entry, 'get', captcha_img_url)
        if captcha_img_response is None or captcha_img_response.status_code != 200:
            entry.fail_with_prefix('Can not get captcha_img')
            return None
        captcha_img = Image.open(BytesIO(captcha_img_response.content))
        captcha_img_hash = toHash(captcha_img)

        for value, answer in answers:
            logger.info((value, answer))
            skip = False
            for q in question_json.values():
                if q['answer'] == answer:
                    skip = True
            if skip:
                logger.info('skip')
                continue
            movies = requests.get(f'https://movie.douban.com/j/subject_suggest?q={answer}',
                                  headers={'user-agent': config.get('user-agent')}).json()
            if len(movies) == 0:
                logger.info(f'https://movie.douban.com/j/subject_suggest?q={answer} length: 0')
            for movie in movies:
                movie_img_response = requests.get(movie.get('img'))
                if movie_img_response is None or movie_img_response.status_code != 200:
                    entry.fail_with_prefix('douban error!')
                    return None
                movie_img = Image.open(BytesIO(movie_img_response.content))
                movie_img_hash = toHash(movie_img)
                score = compareHash(captcha_img_hash, movie_img_hash)
                logger.info(f'{movie.get("title")} url:{movie.get("img")} score: {score}')
                if score > 0.9:
                    question_json[img_name] = {
                        'hash': captcha_img_hash,
                        'answer': answer
                    }
                    question_file.write_text(json.dumps(question_json, indent=4, ensure_ascii=False), encoding='utf-8')
                    return value
                sleep(5)
        logger.info(f'img_name: {captcha_img_url}, answers: {answers}')
        logger.info(json.dumps({
            img_name: {
                'hash': captcha_img_hash,
                'answer': ''
            }
        }, indent=4, ensure_ascii=False))
        entry.fail_with_prefix('Cannot find answer')
        return None

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_nexusphp_messages(entry, config, ignore_title=self.IGNORE_TITLE)

    def handle_hr(self, hr):
        return str(100 - int(hr))


def toHash(img, shape=(10, 10)):
    img = img.resize(shape)
    gray = np.asarray(img.convert('L'))
    s = 0
    hash_str = ''
    for i in range(shape[0]):
        for j in range(shape[1]):
            s = s + gray[i, j]
    avg = s / 100
    for i in range(shape[0]):
        for j in range(shape[1]):
            if gray[i, j] > avg:
                hash_str = hash_str + '1'
            else:
                hash_str = hash_str + '0'
    return hash_str


def compareHash(hash1, hash2, shape=(10, 10)):
    n = 0
    if len(hash1) != len(hash2):
        return -1
    for i in range(len(hash1)):
        if hash1[i] == hash2[i]:
            n = n + 1
    return n / (shape[0] * shape[1])



from typing import Final

from ..core.entry import SignInEntry
from ..base.request import NetworkState
from ..base.request import check_network_state
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.gazelle import Gazelle
from ..utils import net_utils
from ..utils.net_utils import get_module_name


class MainClass(Gazelle):
    URL: Final = 'https://jpopsuki.eu/'
    USER_CLASSES: Final = {
        'uploaded': [26843545600],
        'share_ratio': [1.05],
        'days': [14]
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'cookie': {'type': 'string'},
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            'username': login['username'],
            'password': login['password'],
            'keeplogged': 1,
            'login': 'Log In!',
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/login.php',
                method=self.sign_in_by_login,
                assert_state=(check_network_state, NetworkState.SUCCEED),
                response_urls=['/index.php'],
            ),
        ]

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=[r'<a href="user\.php\?id=\d+" class="username">(.*?)</a>'],
                assert_state=(check_final_state, SignState.SUCCEED),
                use_last_content=True,
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'user_id': 'user.php\\?id=(\\d+)',
            'detail_sources': {
                'default': {
                    'link': '/user.php?id={}',
                    'elements': {
                        'table': '#content > div > div.sidebar > div:nth-last-child(4) > ul',
                        'Community': '#content > div > div.sidebar > div:last-child > ul'
                    }
                }
            }
        })
        return selector



from __future__ import annotations

import re
import threading
from io import BytesIO

from loguru import logger

from ..core.entry import SignInEntry

try:
    from aip import AipOcr
except ImportError:
    AipOcr = None

try:
    from PIL import Image
except ImportError:
    Image = None

qps = 1
lock = threading.Semaphore(qps)


def get_client(entry: SignInEntry, config: dict) -> AipOcr | None:
    if 'aipocr' not in config:
        entry.fail_with_prefix('aipocr not set in config')
        return None

    app_id = config['aipocr'].get('app_id')
    api_key = config['aipocr'].get('api_key')
    secret_key = config['aipocr'].get('secret_key')

    if not (AipOcr and Image):
        entry.fail_with_prefix('Dependency does not exist: [baidu-aip, pillow]')
        return None
    if not (app_id and api_key and secret_key):
        entry.fail_with_prefix('AipOcr not set')
        return None
    return AipOcr(app_id, api_key, secret_key)


def get_jap_ocr(img: Image.Image, entry: SignInEntry, config: dict) -> str | None:
    if not (client := get_client(entry, config)):
        return None
    img_byte_arr = BytesIO()

    if img.mode == "P":
        img = img.convert('RGB')

    img.save(img_byte_arr, format='JPEG')
    try:
        with lock:
            result = client.basicAccurate(img_byte_arr.getvalue(), {'language_type': 'JAP'})
    except Exception as e:
        entry.fail_with_prefix(f'baidu ocr error: {e}')
        return None
    logger.info(result)
    if result.get('error_msg'):
        entry.fail_with_prefix(result.get('error_msg'))
        return None
    text = ''
    for words_list in result.get('words_result'):
        text += words_list.get('words')
    return ''.join(re.findall(r'[\u2E80-\u9FFF]', text))


def get_ocr_code(img: Image.Image, entry: SignInEntry, config: dict) -> tuple:
    if not (client := get_client(entry, config)):
        return None, None

    # transform pixel value < black_threshold to pure black
    black_threshold = 64
    img = img.point(lambda p: 0 if p < black_threshold else p)

    width = img.size[0]
    height = img.size[1]
    for i in range(0, width):
        for j in range(0, height):
            if noise := _detect_noise(img, i, j, width, height):
                img.putpixel((i, j), (255, 255, 255))
    img_byte_arr = BytesIO()
    img.save(img_byte_arr, format='png')
    try:
        with lock:
            # 使用basicGeneral方法，效果更好，专门针对英文验证码
            result = client.basicGeneral(img_byte_arr.getvalue(), {"language_type": "ENG"})
    except Exception as e:
        entry.fail_with_prefix(f'baidu ocr error: {e}')
        return None, None
    logger.info(result)
    if result.get('error_msg'):
        entry.fail_with_prefix(result.get('error_msg'))
        return None, None

    # 检查OCR结果
    if not result.get('words_result') or len(result['words_result']) == 0:
        entry.fail_with_prefix('OCR failed: No text recognized in image')
        return None, img_byte_arr.getvalue()

    code = re.sub('\\W', '', result['words_result'][0]['words'])
    code = code.upper()

    # 验证验证码长度
    if len(code) == 0:
        entry.fail_with_prefix('OCR failed: Empty text recognized')
        return None, img_byte_arr.getvalue()

    return code, img_byte_arr.getvalue()


def _detect_noise(img: Image.Image, i: int, j: int, width: int, height: int) -> bool:
    if i < 25 or i > 122 or j < 15 or j > 24:
        return True
    pixel = img.getpixel((i, j))
    if pixel[0] != 0 or pixel[1] != 0 or pixel[2] != 0:
        return True
    else:
        if i + 1 < width:
            pixel = img.getpixel((i + 1, j))
            if pixel[0] == 0 and pixel[1] == 0 and pixel[2] == 0:
                return False
        if i - 1 > 0:
            pixel = img.getpixel((i - 1, j))
            if pixel[0] == 0 and pixel[1] == 0 and pixel[2] == 0:
                return False
        if j + 1 < height:
            pixel = img.getpixel((i, j + 1))
            if pixel[0] == 0 and pixel[1] == 0 and pixel[2] == 0:
                return False
        if j - 1 > 0:
            pixel = img.getpixel((i, j + 1))
            if pixel[0] == 0 and pixel[1] == 0 and pixel[2] == 0:
                return False
        return True



from typing import Final

from ..core.entry import SignInEntry
# Removed reseed functionality
from ..schema.nexusphp import Attendance
from ..utils import net_utils
from ..utils.value_handler import size


class MainClass(Attendance):
    URL: Final = 'https://ubits.club/'
    IGNORE_TITLE = r'H&R\(ID: \d+\) 已达标'
    USER_CLASSES: Final = {
        'downloaded': [size(1.5, 'TiB'), size(4, 'TiB')],
        'share_ratio': [6, 10],
        'points': [1000000, 5000000],
        'days': [175, 420]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'details': {
                'points': {
                    'regex': r'做种积分.*?([\d,.]+)'
                }
            }
        })
        return selector

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_nexusphp_messages(entry, config, ignore_title=self.IGNORE_TITLE)



from __future__ import annotations

import re

import chardet
from requests import Response


def get_module_name(cls) -> str:
    return cls.__module__.rsplit('.', maxsplit=1)[-1]


def decode(response: Response | None) -> str | None:
    if response is None:
        return None
    content = response.content
    charset_encoding = chardet.detect(content).get('encoding')
    if charset_encoding == 'ascii':
        charset_encoding = 'unicode_escape'
    elif charset_encoding in ['Windows-1254','MacRoman']:
        charset_encoding = 'utf-8'
    return content.decode(charset_encoding if charset_encoding else 'utf-8', 'ignore')


def cookie_str_to_dict(cookie_str: str) -> dict:
    cookie_dict = {}
    for line in cookie_str.split(';'):
        if '=' in line:
            i = line.index('=')
            cookie_dict[line[0:i].strip()] = line[i + 1:].strip()
    return cookie_dict


def cookie_to_str(cookie_items: list) -> str:
    cookie_array = []
    for k, v in cookie_items:
        cookie_array.append(f'{k}={v}')
    return '; '.join(cookie_array).strip()


def dict_merge(dict1: dict, dict2: dict) -> None:
    for i in dict2:
        if isinstance(dict1.get(i), dict) and isinstance(dict2.get(i), dict):
            dict_merge(dict1[i], dict2[i])
        else:
            dict1[i] = dict2[i]


def get_site_name(url: str) -> str:
    if (re_object := re.search('(?<=//).*?(?=/)', url)) and len(domain := re_object.group().split('.')) > 1:
        site_name = domain[len(domain) - 2]
        return site_name if site_name != 'edu' else domain[len(domain) - 3]



"""
执行器模块
重构自原FlexGet插件，移除FlexGet依赖
"""
from __future__ import annotations

import importlib
import json
import pathlib
import pkgutil
import threading
from datetime import datetime
from typing import List

from loguru import logger

from .entry import SignInEntry

lock = threading.Semaphore(1)


def build_sign_in_schema() -> dict:
    """构建签到配置架构"""
    module = None
    sites_schema: dict = {}
    try:
        # 导入基类
        from ..base.sign_in import SignIn

        sites_path = pathlib.Path(__file__).parent.parent / 'sites'
        for module in pkgutil.iter_modules(path=[str(sites_path)]):
            site_class = get_site_class(module.name)
            if issubclass(site_class, SignIn):
                sites_schema.update(site_class.sign_in_build_schema())
    except AttributeError as e:
        logger.error(f"site: {module.name if module else 'unknown'}, error: {e}")
        raise Exception(f"site: {module.name if module else 'unknown'}, error: {e}")
    return sites_schema


def build_sign_in_entry(entry: SignInEntry, config: dict) -> None:
    """构建签到条目"""
    try:
        # 导入基类
        from ..base.sign_in import SignIn

        site_class = get_site_class(entry['class_name'])
        if issubclass(site_class, SignIn):
            site_class.sign_in_build_entry(entry, config)
    except AttributeError as e:
        logger.error(f"site: {entry['site_name']}, error: {e}")
        raise Exception(f"site: {entry['site_name']}, error: {e}")


def save_cookie(entry: SignInEntry) -> None:
    """保存cookie到备份文件"""
    file_name = 'cookies_backup.json'
    site_name = entry['site_name']
    session_cookie = entry.get('session_cookie')
    if not session_cookie:
        return
    with lock:
        # 保存到配置文件目录
        config_dir = entry.get('config', {}).get('config_dir', '.')
        cookies_backup_file = pathlib.Path(config_dir).joinpath(file_name)
        if cookies_backup_file.is_file():
            try:
                cookies_backup_json = json.loads(cookies_backup_file.read_text(encoding='utf-8'))
            except json.JSONDecodeError:
                cookies_backup_json = {}
        else:
            cookies_backup_json = {}
        cookies_backup_json[site_name] = {'date': str(datetime.now().date()), 'cookie': session_cookie}
        cookies_backup_file.write_text(json.dumps(cookies_backup_json, indent=4), encoding='utf-8')


def sign_in(entry: SignInEntry, config: dict) -> SignInEntry:
    """执行签到"""
    try:
        # 导入基类
        from ..base.sign_in import SignIn
        from ..base.message import Message
        from ..base.detail import Detail

        site_class = get_site_class(entry['class_name'])
    except AttributeError as e:
        logger.error(f"site: {entry['class_name']}, error: {e}")
        raise Exception(f"site: {entry['class_name']}, error: {e}")

    site_object = site_class()

    if issubclass(site_class, SignIn):
        entry['prefix'] = 'Sign_in'
        site_object.sign_in(entry, config)
        if entry.failed:
            return entry
        if entry['result']:
            logger.info(f"{entry['title']} {entry['result']}".strip())

    if config.get('get_messages', True) and issubclass(site_class, Message):
        entry['prefix'] = 'Messages'
        site_object.get_messages(entry, config)
        if entry.failed:
            return entry
        if entry['messages']:
            logger.info(f"site_name: {entry['site_name']}, messages: {entry['messages']}")

    if config.get('get_details', True) and issubclass(site_class, Detail):
        entry['prefix'] = 'Details'
        site_object.get_details(entry, config)
        if entry.failed:
            return entry
        if entry['details']:
            logger.info(f"site_name: {entry['site_name']}, details: {entry['details']}")

    if config.get('cookie_backup', True):
        if entry.failed:
            return entry
        save_cookie(entry)
    clean_entry_attr(entry)
    return entry


def clean_entry_attr(entry: SignInEntry) -> None:
    """清理条目属性"""
    for attr in ['base_content', 'prefix']:
        if hasattr(entry, attr):
            delattr(entry, attr)


def get_site_class(class_name: str) -> type:
    """获取站点类"""
    try:
        # 使用绝对导入
        site_module = importlib.import_module(f'pt_checkin.sites.{class_name.lower()}')
        return getattr(site_module, 'MainClass')
    except (ImportError, AttributeError) as e:
        logger.error(f"Failed to import site class {class_name}: {e}")
        raise


def create_sign_in_entries(sites_config: dict, config: dict) -> List[SignInEntry]:
    """创建签到条目列表"""
    entries: List[SignInEntry] = []
    
    for site_name, site_configs in sites_config.items():
        if not isinstance(site_configs, list):
            site_configs = [site_configs]
        
        for sub_site_config in site_configs:
            entry = SignInEntry(
                title=f'{site_name} {datetime.now().date()}',
                url=''
            )
            entry['site_name'] = site_name
            entry['class_name'] = site_name
            entry['site_config'] = sub_site_config
            entry['config'] = config  # 添加全局配置
            entry['result'] = ''
            entry['messages'] = ''
            entry['details'] = ''
            
            try:
                build_sign_in_entry(entry, config)
                entries.append(entry)
            except Exception as e:
                logger.error(f"Failed to build entry for {site_name}: {e}")
                continue
    
    return entries



from typing import Final

from ..core.entry import SignInEntry
from ..schema.nexusphp import NexusPHP


class MainClass(NexusPHP):
    URL: Final = 'https://hdchina.org/'
    SUCCEED_REGEX: Final = '签到已得\\d+魔力值'
    USER_CLASSES: Final = {
        'downloaded': [3298534883328],
        'share_ratio': [4.55],
        'days': [700]
    }

    def sign_in(self, entry: SignInEntry, config: dict) -> None:
        self.sign_in_by_get(entry, config, work_name='attendance')



from typing import Final

from ..core.entry import SignInEntry
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.unit3d import Unit3D
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_join_date


# site_config
#    oneurl: 'xxxxxxxx'
#    cookie: 'xxxxxxxx'

# Choose between oneurl or cookie
# If oneurl provided, cookie will be ignored.

# OneURL is found by accessing your Beyond-HD website,
# hovering over the user icon and going to My Security then going to the One URL (OID) tab,
# and if it's not already active, you need to hit Reset One URL to activate it.
# Then use that link here.

class MainClass(Unit3D):
    URL: Final = 'https://beyond-hd.me/'

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'oneurl': {'type': 'string'},
                    'cookie': {'type': 'string'}
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        site_config = entry['site_config']
        oneurl = site_config.get('oneurl')
        return [
            Work(
                url=oneurl or '/',
                method=self.sign_in_by_get,
                succeed_regex=['<title>BeyondHD | Beyond Your Imagination</title>'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
                response_urls=['https://beyond-hd.me', 'https://beyond-hd.me/']
            )
        ]

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': '/([^.]+\.\d+)">My Profile',
            'detail_sources': {
                'default': {
                    'link': '/{}',
                    'elements': {
                        'bar': '.table-responsive.well-style',
                        'data_table': '.bhd-profile'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': ('(上传|Upload).+?([\\d.]+ ?[ZEPTGMK]?iB)', 2)
                },
                'downloaded': {
                    'regex': ('(下载|Download).+?([\\d.]+ ?[ZEPTGMK]?iB)', 2)
                },
                'share_ratio': {
                    'regex': ('(分享率|Ratio).+?([\\d.]+)', 2)
                },
                'points': {
                    'regex': ('(魔力|BP:).+?(\\d[\\d,. ]*)', 2),
                    'handle': self.handle_points
                },
                'join_date': {
                    'regex': ('(注册日期|Member Since:) (\\d{4}-\\d{2}-\\d{2})', 2),
                    'handle': handle_join_date
                },
                'seeding': {
                    'regex': ('(\d+)\s*?\d*%\s*?(做种|Active Seeds)', 1)
                },
                'leeching': {
                    'regex': ('(\d+)\s*?(吸血|Active Downloads)', 1)
                },
                'hr': {
                    'regex': ('(警告|Warnings).+?(\\d+)', 2)
                }
            }
        }



from typing import Final

# Removed reseed functionalityCookie
from ..schema.nexusphp import AttendanceHR
from ..utils import net_utils
from ..utils.value_handler import size


class MainClass(AttendanceHR):
    URL: Final = 'https://www.pthome.net/'
    USER_CLASSES: Final = {
        'downloaded': [size(2, 'TiB'), size(6, 'TiB')],
        'share_ratio': [1.5, 1.8],
        'points': [600000, 1200000],
        'days': [280, 700]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#info_block > tbody > tr > td > table > tbody > tr > td:nth-child(1)',
                    }
                }
            },
            'details': {
                'points': {
                    'regex': '做种积分([\\d.,]+)',
                }
            }
        })
        return selector



import re
from typing import Final

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.private_torrent import PrivateTorrent
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_infinite


class MainClass(PrivateTorrent):
    URL: Final = 'https://speedapp.io/'
    USER_CLASSES: Final = {
        'uploaded': [109951162777600],
        'share_ratio': [6],
        'days': [2190],
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            '_csrf_token': re.search(r'(?<=name="_csrf_token" value=").+?(?=")', last_content).group(),
            'username': login['username'],
            'password': login['password'],
            '_remember_me': 'on',
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/zh/%E7%99%BB%E5%BD%95?locale=zh',
                method=self.sign_in_by_get,
                assert_state=(check_network_state, NetworkState.SUCCEED),
            ),
            Work(
                url='/zh/登录?locale=zh',
                method=self.sign_in_by_login,
                succeed_regex=['logout'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
                response_urls=['/'],
            )
        ]

    @property
    def details_selector(self) -> dict:
        return {
            'detail_sources': {
                'menu-stats': {
                    'do_not_strip': True,
                    'link': '/profile/menu-stats',
                    'elements': {
                        'all': 'body',
                    }
                },
                'profile': {
                    'link': '/profile',
                    'elements': {
                        'table': '#kt_content > div.d-flex.flex-column-fluid > div > div > div.flex-row-fluid.ml-lg-8'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': r'''(?x)已上传
                                    \s*
                                    ([\d.] + \ [ZEPTGMK] ? B)''',
                },
                'downloaded': {
                    'regex': r'''(?x)已下载
                                    \s*
                                    ([\d.] + \ [ZEPTGMK] ? B)''',
                },
                'share_ratio': {
                    'regex': r'''(?x)比率">
                                    \s*
                                    <i\ class="fas\ fa-fw\ fa-chart-line\ text-info\ fa-sm"></i>
                                    \s*
                                    (Inf. | [\d,.] +)''',
                    'handle': handle_infinite
                },
                'points': {
                    'regex': r'''(?x)奖励积分">
                                    \s*
                                    <i\ class="fas\ fa-fw\ fa-coins\ text-warning\ fa-sm"></i>
                                    \s*
                                    ([\d,.] +)'''
                },
                'join_date': {
                    'regex': r'''(?x)注册日期
                                    \s*
                                    (\d + 年 \d + 月 \d + 日)''',
                    'handle': self.handle_join_date
                },
                'seeding': {
                    'regex': r'''(?x)目前正在播种种子">
                                    \s*
                                    <i\ class="far\ fa-fw\ fa-arrow-alt-circle-up\ text-muted\ fa-sm"></i>
                                    \s*
                                    ([\d,] +)'''
                },
                'leeching': {
                    'regex': r'''(?x)目前正在窃取种子">
                                    \s*
                                    <i\ class="far\ fa-fw\ fa-arrow-alt-circle-down\ text-danger\ text-muted\ fa-sm"></i>
                                    \s*
                                    ([\d,] +)'''
                },
                'hr': {
                    'regex': r'''(?x)HnR
                                    \ 
                                    ([\d,] +)'''
                }
            }
        }

    def handle_join_date(self, value: str) -> str:
        return value.translate(str.maketrans('年月', '--', '日'))



from __future__ import annotations

from typing import Final
from urllib.parse import urljoin

from flexget.utils.soup import get_soup

from ..core.entry import SignInEntry
from ..base.request import NetworkState, check_network_state
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.gazelle import Gazelle
from ..utils import net_utils
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_infinite


class MainClass(Gazelle):
    URL: Final = 'https://gazellegames.net/'
    API_URL: Final = urljoin(URL, '/api.php')
    MESSAGE_URL: Final = urljoin(URL, '/inbox.php?action=viewconv&id={conv_id}')
    USER_CLASSES: Final = {
        'points': [1200, 6000],
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'cookie': {'type': 'string'},
                    'key': {'type': 'string'},
                    'name': {'type': 'string'}
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['Welcome, <a.+?</a>'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'do_not_strip': True,
                    'elements': {
                        'bar': '#community_stats > ul:nth-child(3)',
                        'table': '#content > div > div.sidebar > div.box_main_info',
                        'join_date': '.nobullet span.time'
                    }
                },
                'achievements': {
                    'link': '/user.php?action=achievements',
                    'elements': {
                        'total_point': '#content > div[class=linkbox]'
                    }
                }
            },
            'details': {
                'points': {
                    'regex': 'Total Points: (\\d+)'
                },
                'hr': {
                    'regex': 'Hit \'n\' Runs">(\\d+)'
                },
            }
        })
        return selector

    def get_details(self, entry: SignInEntry, config: dict) -> None:
        site_config = entry['site_config']
        key = site_config.get('key')
        name = site_config.get('name')
        if not (key and name):
            entry.fail_with_prefix('key or name not found')
            return
        params = {
            'request': 'user',
            'key': key,
            'name': name
        }
        details_response_json = self.get_api_response_json(entry, params)
        if not details_response_json:
            return
        entry['details'] = {
            'uploaded': f'{details_response_json.get("response").get("stats").get("uploaded") or 0} B'.replace(',', ''),
            'downloaded': f'{details_response_json.get("response").get("stats").get("downloaded") or 0} B'.replace(',',
                                                                                                                   ''),
            'share_ratio': handle_infinite(
                str(details_response_json.get('response').get('stats').get('ratio') or 0).replace(',', '')),
            'points': str(details_response_json.get('response').get('achievements').get('totalPoints') or 0).replace(
                ',', ''),
            'seeding': str(details_response_json.get('response').get('community').get('seeding') or 0).replace(',', ''),
            'leeching': str(details_response_json.get('response').get('community').get('leeching') or 0).replace(',',
                                                                                                                 ''),
            'hr': str(details_response_json.get('response').get('personal').get('hnrs') or 0).replace(',', '')
        }

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        site_config = entry['site_config']
        key = site_config.get('key')
        if not key:
            entry.fail_with_prefix('key not found')
            return
        params = {
            'request': 'inbox',
            'sort': 'unread',
            'key': key
        }

        messages_response_json = self.get_api_response_json(entry, params)
        if not messages_response_json:
            return
        unread_messages = filter(lambda m: m.get('unread'),
                                 messages_response_json.get('response').get('messages'))
        failed = False
        for message in unread_messages:
            title = message.get('subject')
            conv_id = message.get('convId')
            message_url = MainClass.MESSAGE_URL.format(conv_id=conv_id)
            message_response = self.request(entry, 'get', message_url)
            network_state = check_network_state(entry, message_url, message_response)
            message_body = 'Can not read message body!'
            if network_state != NetworkState.SUCCEED:
                failed = True
            else:
                body_element = get_soup(
                    net_utils.decode(message_response)).select_one('.body')
                if body_element:
                    message_body = body_element.text.strip()
            entry['messages'] = entry['messages'] + (
                f'\nTitle: {title}\nLink: {message_url}\n{message_body}')
        if failed:
            entry.fail_with_prefix('Can not read message body!')

    def get_api_response_json(self, entry: SignInEntry, params: dict) -> dict | None:
        api_response = self.request(entry, 'get', MainClass.API_URL, params=params)
        network_state = check_network_state(entry, api_response.request.url, api_response)
        if network_state != NetworkState.SUCCEED:
            return None
        api_response_json = api_response.json()
        if not api_response_json.get('status') == 'success':
            entry.fail_with_prefix(api_response_json)
            return None
        return api_response_json



from abc import ABC

from .private_torrent import PrivateTorrent
from ..utils.value_handler import handle_infinite


class Ocelot(PrivateTorrent, ABC):

    @property
    def details_selector(self) -> dict:
        return {
            'user_id': r'userdetails\.php\?id=(\d+)',
            'detail_sources': {
                'default': {
                    'link': '/userdetails.php?id={}',
                    'elements': {
                        'bar': '#wrapper > div.mainheader > div > div.statusbar > div:nth-child(2)',
                        'table': '.cblock-content'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': r'Uploaded.+?([\d.]+ ?[ZEPTGMk]?B)'
                },
                'downloaded': {
                    'regex': r'Downloaded.+?([\d.]+ ?[ZEPTGMk]?B)'
                },
                'share_ratio': {
                    'regex': r'Ratio.+?(Inf\.|[\d.]+)',
                    'handle': handle_infinite
                },
                'points': {
                    'regex': (r'Hello.+?\[.+?\][\s\S]+?(\d+)[\s\S]*?(Inf\.|[\d,.]+)', 2)
                },
                'join_date': {
                    'regex': r'Join.date.*?(\d{4}-\d{2}-\d{2})',
                },
                'seeding': {
                    'regex': r'Seeding (\d)+'
                },
                'leeching': None,
                'hr': None
            }
        }



from __future__ import annotations

from typing import Final

# Removed reseed functionality
from ..schema.nexusphp import Attendance
from ..utils import net_utils
from ..utils.value_handler import size


class MainClass(Attendance):
    URL: Final = 'https://www.icc2022.com/'
    USER_CLASSES: Final = {
        'downloaded': [size(750, 'GiB'), size(3, 'TiB')],
        'share_ratio': [3.05, 4.45],
        'points': [400000, 1000000],
        'days': [280, 700]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '.medium',
                        'table': '#outer .embedded'
                    }
                }
            },
            'details': {
                'hr': None
            }
        })
        return selector



from __future__ import annotations

import re
from io import BytesIO
from typing import Final
from urllib.parse import urljoin

from requests import Response

from ..core.entry import SignInEntry
from ..base.request import check_network_state, NetworkState
# Removed reseed functionality
from ..base.sign_in import check_final_state, SignState, check_sign_in_state
from ..base.work import Work
from ..schema.nexusphp import NexusPHP
from ..utils import baidu_ocr
from ..utils import net_utils

try:
    from PIL import Image
except ImportError:
    Image = None


class MainClass(NexusPHP):
    URL: Final = 'https://open.cd/'
    IGNORE_TITLE = '種子被刪除'
    USER_CLASSES: Final = {
        'downloaded': [644245094400, 3298534883328],
        'share_ratio': [3.5, 5],
        'days': [175, 210]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/',
                method=self.sign_in_by_get,
                succeed_regex=['查看簽到記錄'],
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True,
            ),
            Work(
                url='/plugin_sign-in.php',
                method=self.sign_in_by_ocr,
                succeed_regex=['{"state":"success","signindays":"\\d+","integral":"\\d+"}'],
                fail_regex='验证码错误',
                response_urls=['/plugin_sign-in.php', '/plugin_sign-in.php?cmd=signin'],
                assert_state=(check_final_state, SignState.SUCCEED),
            ),
        ]

    def sign_in_by_ocr(self, entry: SignInEntry, config: dict, work: Work, last_content: str) -> Response | None:
        image_hash_response = self.request(entry, 'get', work.url, config)
        image_hash_network_state = check_network_state(entry, work, image_hash_response)
        if image_hash_network_state != NetworkState.SUCCEED:
            entry.fail_with_prefix('Get image hash failed.')
            return None
        # 尝试多种方式获取页面内容
        image_hash_content = None
        try:
            # 首先尝试使用response.text（自动处理编码和压缩）
            image_hash_content = image_hash_response.text
        except Exception:
            # 如果失败，使用net_utils.decode作为备用
            image_hash_content = net_utils.decode(image_hash_response)

        if not image_hash_content:
            entry.fail_with_prefix('Failed to decode response content.')
            return None

        # 检查是否被重定向到登录页面
        login_indicators = ['未登錄!', '錯誤: 該頁面必須在登錄後才能訪問']
        if any(indicator in image_hash_content for indicator in login_indicators):
            entry.fail_with_prefix('Not logged in. Please provide valid cookie.')
            return None

        image_hash_re = re.search('(?<=imagehash=).*?(?=")', image_hash_content)
        img_src_re = re.search('(?<=img src=").*?(?=")', image_hash_content)

        if image_hash_re and img_src_re:
            image_hash = image_hash_re.group()
            img_src = img_src_re.group()

            # 处理HTML实体编码，将&amp;转换为&
            img_src = img_src.replace('&amp;', '&')
            img_url = urljoin(entry['url'], img_src)

            # 验证码图片使用普通requests下载，不使用FlareSolverr和cookie
            import requests
            import os
            from datetime import datetime

            try:
                # 直接使用requests下载图片，不经过FlareSolverr
                headers = {
                    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36',
                    'Referer': 'https://open.cd/plugin_sign-in.php'
                }
                img_response = requests.get(img_url, headers=headers, timeout=30)

                if img_response.status_code != 200:
                    entry.fail_with_prefix(f'Get image failed: HTTP {img_response.status_code}')
                    return None

                # 保存验证码图片到配置文件同级目录
                config_dir = config.get('config_dir', '.')
                timestamp = datetime.now().strftime('%Y%m%d_%H%M%S')
                captcha_filename = f'open_captcha_{timestamp}.png'
                captcha_path = os.path.join(config_dir, captcha_filename)

                # 清理旧的验证码图片
                try:
                    for file in os.listdir(config_dir):
                        if file.startswith('open_captcha_') and file.endswith('.png'):
                            old_path = os.path.join(config_dir, file)
                            os.remove(old_path)
                except Exception as cleanup_e:
                    # 清理失败不影响主流程
                    pass

                # 保存新的验证码图片
                with open(captcha_path, 'wb') as f:
                    f.write(img_response.content)

            except Exception as e:
                entry.fail_with_prefix(f'Get image failed: {e}')
                return None
        else:
            entry.fail_with_prefix(
                'Cannot find key: image_hash. '
                'Page content may have changed or login required.'
            )
            return None

        # 处理图片内容（现在是标准的requests响应）
        try:
            img_content = img_response.content

            # 验证图片数据
            if len(img_content) < 100:  # 图片数据太小，可能有问题
                entry.fail_with_prefix(f'Image data too small: {len(img_content)} bytes')
                return None

            # 检查是否是HTML页面（有时服务器返回错误页面）
            if img_content.startswith(b'<html') or img_content.startswith(b'<!DOCTYPE'):
                entry.fail_with_prefix('Received HTML instead of image data.')
                return None

            img = Image.open(BytesIO(img_content))

        except Exception as e:
            entry.fail_with_prefix(f'Failed to process image: {e}')
            return None
        code, img_byte_arr = baidu_ocr.get_ocr_code(img, entry, config)
        if not entry.failed:
            if len(code) >= 4:  # 验证码长度可能不是固定6位
                # 构建POST URL
                signin_url = f"{work.url}?cmd=signin"

                # 构建POST数据
                data = {
                    'action': 'signin',
                    'imagehash': image_hash,
                    'imagestring': code
                }
                return self.request(
                    entry, 'post', signin_url, config, data=data
                )
        return None

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': (
                            '#info_block > tbody > tr > td > table > '
                            'tbody > tr > td:nth-child(2)'
                        )
                    }
                }
            }
        })
        return selector

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        self.get_nexusphp_messages(
            entry, config, ignore_title=self.IGNORE_TITLE
        )



from __future__ import annotations

from typing import Final
from urllib.parse import urljoin

from ..core.entry import SignInEntry
# Removed reseed functionalityPage
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.nexusphp import NexusPHP
from ..utils import net_utils
from ..utils.net_utils import get_module_name


class MainClass(NexusPHP):
    URL: Final = 'https://hdcity.work/'
    TORRENT_PAGE_URL: Final = urljoin(URL, '/t-{torrent_id}')
    DOWNLOAD_BASE_URL: Final = 'https://assets.hdcity.work/'
    DOWNLOAD_URL_REGEX: Final = '/dl\\.php.*?(?=")'
    USER_CLASSES: Final = {
        'downloaded': [5497558138880, 43980465111040],
        'share_ratio': [2.5, 4],
        'days': [168, 700]
    }

    @classmethod
    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/sign',
                method=self.sign_in_by_get,
                succeed_regex=['本次签到获得魅力\\d+'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'user_id': None,
            'detail_sources': {
                'default': {
                    'link': 'https://hdcity.work/userdetails',
                    'elements': {
                        'bar': '#bottomnav > div.button-group',
                        'table': '.text_alt > table > tbody > tr > td:nth-child(2)'
                    }
                }
            },
            'details': {
                'downloaded': {
                    'regex': 'arrow_downward([\\d.]+ ?[ZEPTGMK]?i?B)'
                },
                'uploaded': {
                    'regex': 'arrow_upward([\\d.]+ ?[ZEPTGMK]?i?B)'
                },
                'share_ratio': None,
                'points': {
                    'regex': '(\\d+)(Bonus|魅力值)'
                },
                'seeding': {
                    'regex': 'play_arrow(\\d+)'
                },
                'leeching': {
                    'regex': 'play_arrow\\d+/(\\d+)'
                },
                'hr': None
            }
        })
        return selector



"""
配置管理器
负责加载和验证配置文件
"""
import pathlib
from typing import Dict, Any

import yaml
from loguru import logger


class ConfigManager:
    """配置管理器"""
    
    def __init__(self, config_path: str = 'config.yml'):
        self.config_path = pathlib.Path(config_path)
        self.config_dir = self.config_path.parent  # 配置文件所在目录
        self.config: Dict[str, Any] = {}
        self.load_config()
    
    def load_config(self) -> None:
        """加载配置文件"""
        if not self.config_path.exists():
            logger.error(f"配置文件不存在: {self.config_path}")
            raise FileNotFoundError(f"配置文件不存在: {self.config_path}")
        
        try:
            with open(self.config_path, 'r', encoding='utf-8') as f:
                self.config = yaml.safe_load(f) or {}
            logger.info(f"配置文件加载成功: {self.config_path}")
            logger.info(f"FlareSolverr配置: {self.config.get('flaresolverr', 'Not found')}")
            logger.info(f"站点配置: {list(self.config.get('sites', {}).keys())}")
            self._validate_config()
        except yaml.YAMLError as e:
            logger.error(f"配置文件格式错误: {e}")
            raise
        except Exception as e:
            logger.error(f"加载配置文件失败: {e}")
            raise
    
    def _validate_config(self) -> None:
        """验证配置文件"""
        # 设置默认值
        self.config.setdefault('max_workers', 1)
        self.config.setdefault('user_agent', 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36')
        self.config.setdefault('get_messages', True)
        self.config.setdefault('get_details', True)
        self.config.setdefault('cookie_backup', True)
        self.config.setdefault('schedule_time', '08:30')
        self.config.setdefault('sites', {})
        
        # 验证必要配置
        if not self.config.get('sites'):
            logger.warning("未配置任何站点")
        
        # 验证站点配置格式
        sites = self.config.get('sites', {})
        for site_name, site_config in sites.items():
            if isinstance(site_config, str):
                # 简单cookie配置，转换为标准格式
                self.config['sites'][site_name] = {'cookie': site_config}
            elif isinstance(site_config, dict):
                # 详细配置，保持不变
                pass
            else:
                logger.warning(f"站点 {site_name} 配置格式不正确")
    
    def get(self, key: str, default: Any = None) -> Any:
        """获取配置项"""
        return self.config.get(key, default)
    
    def get_sites(self) -> Dict[str, Any]:
        """获取站点配置"""
        return self.config.get('sites', {})
    
    def get_user_agent(self) -> str:
        """获取User-Agent"""
        return self.config.get('user_agent', 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36')
    
    def get_max_workers(self) -> int:
        """获取最大工作线程数"""
        return self.config.get('max_workers', 1)
    
    def get_schedule_time(self) -> str:
        """获取调度时间"""
        return self.config.get('schedule_time', '08:30')

    def get_max_failed_attempts(self) -> int:
        """获取最大失败次数"""
        return self.config.get('max_failed_attempts', 3)

    def get_failed_retry_interval(self) -> int:
        """获取失败重试间隔（小时）"""
        return self.config.get('failed_retry_interval', 2)
    
    def get_baidu_ocr_config(self) -> Dict[str, str]:
        """获取百度OCR配置
        统一使用新格式 aipocr: {app_id, api_key, secret_key}
        """
        if isinstance(self.config.get('aipocr'), dict):
            a = self.config['aipocr']
            return {
                'app_id': a.get('app_id', ''),
                'api_key': a.get('api_key', ''),
                'secret_key': a.get('secret_key', ''),
            }
        # 如果没有新格式配置，返回空配置
        return {
            'app_id': '',
            'api_key': '',
            'secret_key': ''
        }
    
    def prepare_config_for_executor(self) -> Dict[str, Any]:
        """为执行器准备配置"""
        return {
            'user-agent': self.get_user_agent(),
            'max_workers': self.get_max_workers(),
            'get_messages': self.get('get_messages', True),
            'get_details': self.get('get_details', True),
            'cookie_backup': self.get('cookie_backup', True),
            'aipocr': self.get_baidu_ocr_config(),
            'flaresolverr': self.config.get('flaresolverr', {}),
            'config_dir': str(self.config_dir),  # 配置文件目录路径
            'sites': self.get_sites()
        }



from typing import Final

# Removed reseed functionality
from ..schema.nexusphp import Visit
from ..utils import net_utils


class MainClass(Visit):
    URL: Final = 'https://nanyangpt.com/'
    SUCCEED_REGEX: Final = '魔力豆 \\(.*?\\)'
    USER_CLASSES: Final = {
        'downloaded': [536870912000, 1099511627776],
        'share_ratio': [5.5, 8.5],
        'days': [140, 350]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#userlink > ul > div:nth-child(3)'
                    }
                }
            }
        })
        return selector



from __future__ import annotations

import datetime
import json
from typing import Final
from urllib.parse import urljoin

from dateutil.parser import parse
from flexget.entry import Entry
from requests import Response, request

from ..core.entry import SignInEntry
from ..base.request import NetworkState
# Removed reseed functionality
from ..base.sign_in import check_final_state, SignState, Work
from ..schema.nexusphp import NexusPHP
from ..utils import url_recorder
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_infinite


class MainClass(NexusPHP, Reseed):
    URL: Final = 'https://api.m-team.io/'
    PROFILE_URL = '/api/member/profile'
    MY_PEER_STATUS = '/api/tracker/myPeerStatus'
    GEN_DL_TOKEN = '/api/torrent/genDlToken'
    SUCCEED_REGEX = 'SUCCESS'
    USER_CLASSES: Final = {
        'downloaded': [2147483648000, 3221225472000],
        'share_ratio': [7, 9],
        'days': [168, 224]
    }

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'key': {'type': 'string'}
                },
                'additionalProperties': False
            }
        }

    def get_details(self, entry: SignInEntry, config: dict) -> None:
        details_response_json = json.loads(entry['base_content'])
        if not details_response_json:
            return
        my_peer_status_response = self.request(entry, 'POST', urljoin(self.URL, self.MY_PEER_STATUS))
        my_peer_status_response_json = my_peer_status_response.json()
        entry['details'] = {
            'uploaded': f'{details_response_json.get("data").get("memberCount").get("uploaded") or 0} B'.replace(',',
                                                                                                                 ''),
            'downloaded': f'{details_response_json.get("data").get("memberCount").get("downloaded") or 0} B'.replace(
                ',',
                ''),
            'share_ratio': handle_infinite(
                str(details_response_json.get('data').get('memberCount').get('shareRate') or 0).replace(',', '')),
            'points': str(details_response_json.get('data').get('memberCount').get('bonus') or 0).replace(
                ',', ''),
            'seeding': str(my_peer_status_response_json.get('data').get('seeder') or 0).replace(',', ''),
            'leeching': str(my_peer_status_response_json.get('data').get('leecher') or 0).replace(',',
                                                                                                  ''),
            'hr': '*'
        }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url=self.PROFILE_URL,
                method=self.sign_in_by_post,
                data={},
                succeed_regex=[self.SUCCEED_REGEX],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    def sign_in_by_post(self,
                        entry: SignInEntry,
                        config: dict,
                        work: Work,
                        last_content: str = None,
                        ) -> Response | None:
        response = super().sign_in_by_post(entry, config, work, last_content)
        response_json = response.json()
        last_browse = response_json.get('data').get('memberStatus').get('lastBrowse')
        target_date = datetime.datetime.strptime(last_browse, '%Y-%m-%d %H:%M:%S')
        days = (datetime.datetime.now() - target_date).days
        if days > 29:
            entry.fail_with_prefix(f'last browse: {days} days, {self.URL}')
            return
        return response

    def request(self,
                entry: SignInEntry,
                method: str,
                url: str,
                **kwargs,
                ) -> Response | None:
        key = entry.get('site_config').get('key')
        try:
            return super().request(entry, 'POST', url, headers={'x-api-key': key})
        except Exception as e:
            entry.fail_with_prefix(NetworkState.NETWORK_ERROR.value.format(url=url, error=e))

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        return

    @classmethod
    


from typing import Final

from ..core.entry import SignInEntry
# Removed reseed functionality
from ..base.sign_in import SignState
from ..base.sign_in import check_final_state
from ..base.work import Work
from ..schema.nexusphp import Visit
from ..utils import net_utils


class MainClass(Visit):
    URL: Final = 'https://byr.pt/'
    USER_CLASSES: Final = {
        'uploaded': [4398046511104, 140737488355328],
        'share_ratio': [3.05, 4.55],
        'days': [168, 336]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/index.php',
                method=self.sign_in_by_get,
                succeed_regex=['欢迎'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#info_block td.bottom[style*=left]',
                    }
                }
            }
        })
        return selector



from typing import Final

from ..core.entry import SignInEntry
# Removed reseed functionality
from ..base.sign_in import check_sign_in_state, SignState, check_final_state, Work
from ..schema.nexusphp import NexusPHP
from ..utils import net_utils


class MainClass(NexusPHP):
    URL: Final = 'https://www.haidan.video/'
    USER_CLASSES: Final = {
        'downloaded': [2199023255552, 8796093022208],
        'share_ratio': [4, 5.5],
        'days': [175, 364]
    }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/index.php',
                method=self.sign_in_by_get,
                succeed_regex=['(?<=value=")已经打卡(?=")'],
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True
            ),
            Work(
                url='/signin.php',
                method=self.sign_in_by_get,
                succeed_regex=['(?<=value=")已经打卡(?=")'],
                response_urls=['/signin.php', '/index.php'],
                assert_state=(check_final_state, SignState.SUCCEED)
            )
        ]

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': '#head > div.top-panel.special-border > div > div:nth-child(2)',
                        'table': 'body > div.mainroute > div.mainpanel.special-border > table > tbody > tr > td > table'
                    }
                }
            },
            'details': {
                'hr': None
            }
        })
        return selector



from typing import Final

from ..schema.nexusphp import Visit


class MainClass(Visit):
    URL: Final = 'https://pt.itzmx.com/'
    USER_CLASSES: Final = {
        'downloaded': [2147483648000, 1759218604442],
        'share_ratio': [4, 6],
        'days': [70, 1092]
    }



from typing import Final

# Removed reseed functionality
from ..schema.nexusphp import Visit


class MainClass(Visit):
    URL: Final = 'https://www.dragonhd.xyz/'
    USER_CLASSES: Final = {
        'downloaded': [805306368000, 3298534883328],
        'share_ratio': [3.05, 4.55],
        'days': [280, 700]
    }



from __future__ import annotations

import re
from abc import ABC, abstractmethod
from urllib.parse import urljoin

from bs4 import BeautifulSoup
from loguru import logger
from requests import Response

from ..base.detail import Detail
from ..core.entry import SignInEntry
from ..base.message import Message
from ..base.request import Request, NetworkState, check_network_state
from ..base.sign_in import Work, check_state, SignIn
from ..utils import net_utils
from ..utils.net_utils import get_module_name


class PrivateTorrent(Request, SignIn, Detail, Message, ABC):
    @property
    @abstractmethod
    def URL(self) -> str:
        pass

    USER_CLASSES = {}

    DOWNLOAD_PAGE_TEMPLATE = 'download.php?id={torrent_id}'

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {get_module_name(cls): {'type': 'string'}}

    @classmethod
    def sign_in_build_entry(cls, entry: SignInEntry, config: dict) -> None:
        entry['url'] = cls.URL
        site_config: str | dict = entry['site_config']
        headers: dict = {
            'user-agent': config.get('user-agent'),
            'referer': entry['url'],
            'accept-encoding': 'gzip, deflate, br',
        }
        cookie: str | None = None
        if isinstance(site_config, str):
            cookie = site_config
        elif isinstance(site_config, dict):
            cookie = site_config.get('cookie')
        if cookie:
            entry['cookie'] = cookie
        entry['headers'] = headers
        entry['user_classes'] = cls.USER_CLASSES

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {}

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return []

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return []

    def sign_in(self, entry: SignInEntry, config: dict) -> None:
        workflow: list[Work] = []
        if not entry.get('cookie'):
            workflow.extend(self.sign_in_build_login_workflow(entry, config))
        workflow.extend(self.sign_in_build_workflow(entry, config))
        if not entry.get('url') or not workflow:
            entry.fail_with_prefix(f"site: {entry['site_name']} url or workflow is empty")
            return
        last_work: Work | None = None
        last_response: Response | None = None
        last_content: str | None = None
        for work in workflow:
            work.url = urljoin(entry['url'], work.url)
            work.response_urls = list(map(lambda response_url: urljoin(entry['url'], response_url), work.response_urls))

            if work.use_last_content and last_work:
                work.response_urls = last_work.response_urls
            else:
                last_response = work.method(entry, config, work, last_content)
                last_content = net_utils.decode(last_response)

            if work.is_base_content:
                entry['base_content'] = last_content
            if not check_state(entry, work, last_response, last_content):
                return
            last_work = work

    def get_messages(self, entry: SignInEntry, config: dict) -> None:
        entry['result'] += '(TODO: Message)'

    @property
    def details_selector(self) -> dict:
        return {}

    def get_user_id(self,
                    entry: SignInEntry,
                    user_id_selector: str,
                    base_content: str
                    ) -> str | None:
        if user_id_match := re.search(user_id_selector, base_content):
            return user_id_match.group(1)
        entry.fail_with_prefix('User id not found.')
        logger.error(f'site: {entry["site_name"]} User id not found. content: {base_content}')
        return None

    def get_detail_value(self, content: str, detail_config: dict) -> str | None:
        if detail_config is None:
            return '*'
        regex: str | tuple = detail_config['regex']
        group_index = 1
        if isinstance(regex, tuple):
            regex, group_index = regex
        if not (detail_match := re.search(regex, content, re.DOTALL)):
            return None
        if not (detail := detail_match.group(group_index)):
            return None
        if handle := detail_config.get('handle'):
            detail = handle(detail)
        if type(detail) == str:
            detail = detail.replace(',', '')
        return str(detail)

    def get_details_base(self, entry: SignInEntry, config, selector: dict) -> None:
        if not (base_content := entry.get('base_content')):
            entry.fail_with_prefix('base_content is None.')
            return
        user_id = ''
        if (user_id_selector := selector.get('user_id')) and not (
                user_id := self.get_user_id(entry, user_id_selector, base_content)):
            return
        details_text = ''
        for detail_source in selector['detail_sources'].values():
            if detail_source.get('link'):
                detail_source['link'] = urljoin(entry['url'], detail_source['link'].format(user_id))
                detail_response = self.request(entry, 'get', detail_source['link'])
                network_state = check_network_state(entry, detail_source['link'], detail_response)
                if network_state != NetworkState.SUCCEED:
                    return
                detail_content = net_utils.decode(detail_response)
            else:
                detail_content = base_content
            if elements := detail_source.get('elements'):
                soup = BeautifulSoup(detail_content, "html.parser")
                for name, sel in elements.items():
                    if sel:
                        if details_info := soup.select_one(sel):
                            details_text += str(details_info) if detail_source.get(
                                'do_not_strip') else details_info.text
                        else:
                            entry.fail_with_prefix(f'Element: {name} not found.')
                            logger.error('site: {} element: {} not found, selector: {}, soup: {}',
                                         entry['site_name'],
                                         name, sel, soup)
                            return
            else:
                details_text += detail_content
        if not details_text:
            entry.fail_with_prefix('details_text is None.')
            return
        logger.debug(details_text)
        details = {}
        for detail_name, detail_config in selector['details'].items():
            if not (detail_value := self.get_detail_value(details_text, detail_config)):
                entry.fail_with_prefix(f'detail: {detail_name} not found.')
                logger.error(
                    f"Details=> site: {entry['site_name']}, regex: {detail_config['regex']}，details_text: {details_text}")
                return
            details[detail_name] = detail_value
        if isinstance(entry['site_config'], dict):
            for key, value in details.items():
                if value == '*':
                    details[key] = entry['site_config'].get(key, '*')
        entry['details'] = details

    def get_details(self, entry: SignInEntry, config: dict) -> None:
        self.get_details_base(entry, config, self.details_selector)

    def sign_in_by_get(self,
                       entry: SignInEntry,
                       config: dict,
                       work: Work,
                       last_content: str = None,
                       ) -> Response | None:
        return self.request(entry, 'get', work.url, config)

    def sign_in_by_post(self,
                        entry: SignInEntry,
                        config: dict,
                        work: Work,
                        last_content: str = None,
                        ) -> Response | None:
        data = {}
        for key, regex in work.data.items():
            if key == 'fixed':
                net_utils.dict_merge(data, regex)
            elif value_search := re.search(regex, last_content):
                data[key] = value_search.group()
            else:
                entry.fail_with_prefix(f'Cannot find key: {key}, url: {work.url}')
                return None
        return self.request(entry, 'post', work.url, config, data=data)

    def sign_in_by_login(self,
                         entry: SignInEntry,
                         config: dict,
                         work: Work,
                         last_content: str,
                         ) -> Response | None:
        if not (login := entry['site_config'].get('login')):
            entry.fail_with_prefix('Login data not found!')
            return None
        return self.request(entry, 'post', work.url, config,
                           data=self.sign_in_build_login_data(login, last_content))



from __future__ import annotations

import itertools
import re
import time
from io import BytesIO
from pathlib import Path
from typing import Final
from urllib.parse import urljoin

from requests import Response

from ..core.entry import SignInEntry
from ..base.request import NetworkState, check_network_state
from ..base.sign_in import SignState, check_sign_in_state, check_final_state
from ..base.work import Work
from ..utils import net_utils, baidu_ocr, dmhy_image
from ..utils.net_utils import get_module_name

try:
    from fuzzywuzzy import fuzz, process
except ImportError:
    fuzz = None
    process = None

from loguru import logger

from ..schema.nexusphp import NexusPHP

try:
    from PIL import Image
except ImportError:
    Image = None

_RETRY = 20
_CHAR_COUNT = 4
_SCORE = 40


class MainClass(NexusPHP):
    URL: Final = 'https://u2.dmhy.org/'
    USERNAME_REGEX: Final = '<bdo dir=\'ltr\'>{username}</bdo>'
    SUCCEED_REGEX: Final = '.{0,500}奖励UCoin: <b>\\d+'
    USER_CLASSES: Final = {
        'downloaded': [3298534883328],
        'share_ratio': [4.55],
        'days': [700]
    }

    DATA = {
        'regex_keys': ['<input type="submit" name="(captcha_.*?)" value="(.*?)" />'],
        'req': '<input type="hidden" name="req" value="(.*?)" />',
        'hash': '<input type="hidden" name="hash" value="(.*?)" />',
        'form': '<input type="hidden" name="form" value="(.*?)" />'
    }

    def __init__(self):
        super().__init__()
        self.times = 0

    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'username': {'type': 'string'},
                    'cookie': {'type': 'string'},
                    'comment': {'type': 'string'},
                    'ocr_config': {
                        'type': 'object',
                        'properties': {
                            'retry': {'type': 'integer'},
                            'char_count': {'type': 'integer'},
                            'score': {'type': 'integer'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        site_config = entry['site_config']
        succeed_regex = [self.USERNAME_REGEX.format(username=site_config.get('username')) + self.SUCCEED_REGEX,
                         '<a href="showup.php">已[签簽]到</a>']
        return [
            Work(
                url='/showup.php?action=show',
                method=self.sign_in_by_get,
                succeed_regex=succeed_regex,
                assert_state=(check_sign_in_state, SignState.NO_SIGN_IN),
                is_base_content=True
            ),
            Work(
                url='/showup.php?action=show',
                method=self.sign_in_by_anime,
                data=self.DATA,
                assert_state=(check_network_state, NetworkState.SUCCEED),
                img_regex='image\\.php\\?action=adbc2&req=.+?(?=&imagehash)',
                reload_regex='image\\.php\\?action=reload_adbc2&div=showup&rand=\\d+'
            ),
            Work(
                url='/showup.php?action=show',
                method=self.sign_in_by_get,
                succeed_regex=succeed_regex,
                fail_regex='这是一个杯具。<br />验证码已过期。',
                assert_state=(check_final_state, SignState.SUCCEED)
            )
        ]

    def sign_in_by_anime(self, entry: SignInEntry, config: dict, work: Work, last_content: str) -> Response | None:
        if not fuzz or not process:
            entry.fail_with_prefix('Dependency does not exist: [fuzzywuzzy]')
            return None

        ocr_config = entry['site_config'].get('ocr_config', {})
        ocr_config.setdefault('retry', _RETRY)
        ocr_config.setdefault('char_count', _CHAR_COUNT)
        ocr_config.setdefault('score', _SCORE)

        if not (data := self.build_data(entry, config, work, last_content, ocr_config)):
            entry.fail_with_prefix('Maximum number of retries reached' if self.times == ocr_config.get('retry')
                                   else 'Can not build_data')
            return None
        logger.info(data)
        return self.request(entry, 'post', work.url, data=data)

    def build_data(self, entry: SignInEntry, config: dict, work: Work, base_content: str,
                   ocr_config: dict) -> dict | None:
        if entry.failed:
            return None
        if not (img_url_match := re.search(work.img_regex, base_content)):
            entry.fail_with_prefix('Can not found img_url')
            return None
        img_url = img_url_match.group()
        logger.debug('attempts: {} / {}, url: {}', self.times, ocr_config.get('retry'), urljoin(entry['url'], img_url))
        data = {}
        found = False
        if images := self.get_image(entry, config, img_url, ocr_config.get('char_count')):
            image1, image2 = images
            self.save_iamge(image1, 'step3_a_diff.png')
            self.save_iamge(image2, 'step3_b_diff.png')
            ocr_text1 = baidu_ocr.get_jap_ocr(image1, entry, config)
            ocr_text2 = baidu_ocr.get_jap_ocr(image2, entry, config)
            if entry.failed:
                return None
            oct_text = ocr_text1 if len(ocr_text1) > len(ocr_text2) else ocr_text2
            logger.debug('jap_ocr: {}', oct_text)
            if oct_text and len(oct_text) > ocr_config['char_count']:
                for key, regex in work.data.items():
                    if key == 'regex_keys':
                        for regex_key in regex:
                            select = {}
                            ratio_score = 0
                            if not (regex_key_search := re.findall(regex_key, base_content, re.DOTALL)):
                                entry.fail_with_prefix(
                                    'Cannot find regex_key: {}, url: {}'.format(regex_key, work.url))
                                return None
                            for captcha, value in regex_key_search:
                                if answer_list := list(filter(lambda x2: len(x2) > 0,
                                                              map(lambda x: ''.join(re.findall(r'[\u2E80-\u9FFF]', x)
                                                                                    ), value.split('\n')))):
                                    split_value, partial_ratio = process.extractOne(oct_text, answer_list,
                                                                                    scorer=fuzz.partial_ratio)
                                else:
                                    partial_ratio = 0
                                if partial_ratio > ratio_score:
                                    select = (captcha, value)
                                    ratio_score = partial_ratio
                                logger.debug('value: {}, ratio: {}', value.replace('\n', '\\'), partial_ratio)
                            if ratio_score and ratio_score > ocr_config['score']:
                                captcha, value = select
                                data[captcha] = value
                                found = True
                    else:
                        if not (value_search := re.search(regex, base_content, re.DOTALL)):
                            entry.fail_with_prefix('Cannot find key: {}, url: {}'.format(key, work.url))
                            return None
                        data[key] = value_search.group(1)

        if not found:
            if self.times >= ocr_config.get('retry'):
                return None
            self.times += 1
            reload_url = re.search(work.reload_regex, base_content).group()
            real_reload_url = urljoin(entry['url'], reload_url)
            reload_response = self.request(entry, 'get', real_reload_url)
            reload__net_state = check_network_state(entry, real_reload_url, reload_response)
            if reload__net_state != NetworkState.SUCCEED:
                return None
            reload_content = net_utils.decode(reload_response)
            return self.build_data(entry, config, work, reload_content, ocr_config)
        site_config = entry['site_config']
        data['message'] = site_config.get('comment')
        return data

    def get_image(self, entry: SignInEntry, config: dict, img_url: str, char_count: int) -> tuple | None:
        image_list = []
        checked_list = []
        images_sort_match = None
        new_image = self.get_new_image(entry, img_url)
        if not dmhy_image.check_analysis(new_image):
            self.save_iamge(new_image, 'z_failed.png')
            logger.debug('can not analyzed!')
            return None
        original_text = baidu_ocr.get_jap_ocr(new_image, entry, config)
        logger.debug('original_ocr: {}', original_text)
        if original_text is None or len(original_text) < char_count:
            return None
        image_list.append(new_image)
        while not images_sort_match and len(image_list) < 8:
            if not (new_image := self.get_new_image(entry, img_url)):
                return None
            image_list.append(new_image)
            for images in list(itertools.combinations(image_list, 2)):
                if images not in checked_list:
                    checked_list.append(images)
                    image1, image2 = images
                    self.save_iamge(image1, 'step1_a_original.png')
                    self.save_iamge(image2, 'step1_b_original.png')
                    if dmhy_image.compare_images_sort(image1, image2):
                        images_sort_match = images
                        break
        if images_sort_match:
            image1, image2 = images_sort_match
            image_a_split_1, image_a_split_2 = dmhy_image.split_image(image1)
            self.save_iamge(image_a_split_1, 'step2_a_split_1.png')
            self.save_iamge(image_a_split_2, 'step2_a_split_2.png')
            image_b_split_1, image_b_split_2 = dmhy_image.split_image(image2)
            self.save_iamge(image_b_split_1, 'step2_b_split_1.png')
            self.save_iamge(image_b_split_2, 'step2_b_split_2.png')
            image_last = dmhy_image.compare_images(image_a_split_1, image_b_split_1)
            image_last2 = dmhy_image.compare_images(image_a_split_2, image_b_split_2)
            if image_last and not image_last2:
                self.save_iamge(image_last[0], 'step3_a_split_1_diff.png')
                self.save_iamge(image_last[1], 'step3_b_split_1_diff.png')
                self.save_iamge(image_last[2], 'step4_split_1_diff.png')
                question_image = (image_last[0], image_last[1])
            elif image_last2 and not image_last:
                self.save_iamge(image_last2[0], 'step3_a_split_2_diff.png')
                self.save_iamge(image_last2[1], 'step3_b_split_2_diff.png')
                self.save_iamge(image_last2[2], 'step4_split_2_diff.png')
                question_image = (image_last2[0], image_last2[1])
            elif image_last:
                logger.debug('compare_images: Two identical are returned')
                self.save_iamge(image1, 'identical.png')
                self.save_iamge(image2, 'identical.png')
                self.save_iamge(image_last[0], 'identical_a_split_1_diff.png')
                self.save_iamge(image_last[1], 'identical_b_split_1_diff.png')
                self.save_iamge(image_last[2], 'identical_split_1_diff.png')
                self.save_iamge(image_last2[0], 'identical_a_split_2_diff.png')
                self.save_iamge(image_last2[1], 'identical_b_split_2_diff.png')
                self.save_iamge(image_last2[2], 'identical_split_2_diff.png')
                return None
            else:
                logger.debug('compare_images: no Content')
                return None
            return question_image
        return None

    def get_new_image(self, entry: SignInEntry, img_url: str) -> Image.Image | None:
        time.sleep(3)
        logger.debug('request image...')
        real_img_url = urljoin(entry['url'], img_url)
        base_img_response = self.request(entry, 'get', real_img_url)
        if base_img_response is None or base_img_response.status_code != 200 or base_img_response.url == urljoin(
                entry['url'], '/pic/trans.gif?debug=NIM'):
            return None
        new_image = Image.open(BytesIO(base_img_response.content))
        dmhy_image.remove_date_string(new_image)
        return new_image

    def save_iamge(self, new_image: Image.Image | None, path: str) -> None:
        if not Path('dmhy').is_dir():
            return
        if new_image:
            new_image.save('dmhy/' + path)

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'details': {
                'uploaded': {
                    'regex': r'上传量:  ([\d.]+ [ZEPTGMK]?i?B)'
                    # 'handle': self.handle_suffix
                },
                'downloaded': {
                    'regex': r'下载量:  ([\d.]+ [ZEPTGMK]?i?B)'
                    # 'handle': self.handle_suffix
                },
                'points': {
                    'regex': r'UCoin.*?\d+\(([\d,.]+)'
                },
                'seeding': {
                    'regex': r'客户端.*?: \d+ \((\d+)'
                },
                'leeching': {
                    'regex': r'客户端.*?: \d+ \(\d+ (\d+)'
                },
                'hr': None
            }
        })
        return selector

    # def handle_suffix(self, value: str):
    #     return value.replace('烫', 'TiB')


from typing import Final

# Removed reseed functionality
from ..schema.nexusphp import Visit
from ..utils import net_utils


class MainClass(Visit):
    URL: Final = 'https://pt.sjtu.edu.cn/'
    SUCCEED_REGEX: Final = '魔力值 \\(\\d+\\)'
    USER_CLASSES: Final = {
        'downloaded': [805306368000, 3298534883328],
        'share_ratio': [3.05, 4.55],
        'days': [280, 700]
    }

    @property
    def details_selector(self) -> dict:
        selector = super().details_selector
        net_utils.dict_merge(selector, {
            'detail_sources': {
                'default': {
                    'elements': {
                        'bar': None,
                        'table': 'table[border="1"]'
                    }
                },
                'extend': {
                    'link': '/viewpeerstatus.php',
                    'elements': {
                        'bar': 'li'
                    },
                    'do_not_strip': True
                }
            }
        })
        return selector



from abc import ABC

from .private_torrent import PrivateTorrent
from ..core.entry import SignInEntry
from ..base.sign_in import check_final_state, SignState
from ..base.work import Work
from ..utils.net_utils import get_module_name
from ..utils.value_handler import handle_infinite


class XWT(PrivateTorrent, ABC):
    @classmethod
    def sign_in_build_schema(cls) -> dict:
        return {
            get_module_name(cls): {
                'type': 'object',
                'properties': {
                    'login': {
                        'type': 'object',
                        'properties': {
                            'username': {'type': 'string'},
                            'password': {'type': 'string'}
                        },
                        'additionalProperties': False
                    }
                },
                'additionalProperties': False
            }
        }

    def sign_in_build_login_workflow(self, entry: SignInEntry, config: dict) -> list[Work]:
        return [
            Work(
                url='/takelogin.php',
                method=self.sign_in_by_login,
                succeed_regex=[r'Top 5 Torrents'],
                assert_state=(check_final_state, SignState.SUCCEED),
                is_base_content=True,
                response_urls=['/']
            )
        ]

    def sign_in_build_login_data(self, login: dict, last_content: str) -> dict:
        return {
            'username': login['username'],
            'password': login['password'],
            'returnto': '/'
        }

    @property
    def details_selector(self) -> dict:
        return {
            'detail_sources': {
                'default': {
                    'link': '/browse.php',
                    'elements': {
                        'ratio': '#wel-radio',
                        'up': '#wel-radiok',
                        'down': '#wel-radio2',
                        'active': '#wel-radio3'
                    }
                }
            },
            'details': {
                'uploaded': {
                    'regex': r'Up:\s*([\d.]+ (?i:[ZEPTGMK])B)'
                },
                'downloaded': {
                    'regex': r'(?i)Down:\s*([\d.]+ [ZEPTGMK]B)'
                },
                'share_ratio': {
                    'regex': r'Ratio:\s*(---|[\d,.]+)',
                    'handle': handle_infinite
                },
                'points': None,
                'seeding': {
                    'regex': r'Active:\s*(\d+)'
                },
                'leeching': {
                    'regex': r'Active:\s*\d+\s*(\d+)'
                },
                'hr': None
            }
        }



"""
任务调度器
负责定时执行签到任务
"""
import time
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
from typing import List

import schedule
from loguru import logger

from .config_manager import ConfigManager
from .entry import SignInEntry
from .signin_status import SignInStatusManager


class TaskScheduler:
    """任务调度器"""
    
    def __init__(self, config_manager: ConfigManager):
        self.config_manager = config_manager
        # 状态文件保存到配置文件同级目录
        status_file = config_manager.config_dir / 'signin_status.json'
        self.status_manager = SignInStatusManager(str(status_file))
        self.running = False
    
    def setup_schedule(self) -> None:
        """设置调度任务"""
        schedule_time = self.config_manager.get_schedule_time()
        schedule.every().day.at(schedule_time).do(self.run_sign_in_task)
        logger.info(f"已设置每日 {schedule_time} 执行签到任务")
    
    def run_sign_in_task(self, force_options: dict = None) -> None:
        """执行签到任务"""
        logger.info("开始执行签到任务")
        start_time = datetime.now()

        if force_options is None:
            force_options = {'force_all': False, 'force_site': None}

        try:
            # 动态导入避免循环导入
            from .executor import create_sign_in_entries, sign_in
            # 移除notify导入，使用内置日志记录

            # 准备配置
            config = self.config_manager.prepare_config_for_executor()
            sites_config = self.config_manager.get_sites()

            if not sites_config:
                logger.warning("未配置任何站点，跳过签到任务")
                return

            # 创建签到条目
            entries = create_sign_in_entries(sites_config, config)
            if not entries:
                logger.warning("未创建任何签到条目")
                return
            
            logger.info(f"创建了 {len(entries)} 个签到条目")
            
            # 过滤今日条目和已签到条目
            date_now = str(datetime.now().date())
            valid_entries = []
            skipped_entries = []

            # 获取失败次数限制配置
            max_failed_attempts = self.config_manager.get_max_failed_attempts()
            retry_interval = self.config_manager.get_failed_retry_interval()

            for entry in entries:
                if date_now not in entry['title']:
                    logger.debug(f"{entry['title']} 不是今日条目，跳过")
                    continue

                site_name = entry['site_name']

                # 如果指定了特定站点，只处理指定的站点
                force_sites = force_options.get('force_sites', [])
                if force_sites and site_name not in force_sites:
                    logger.debug(f"跳过未指定站点: {site_name}")
                    continue

                # 检查是否需要强制签到
                force_sites = force_options.get('force_sites', [])
                should_force = force_options.get('force_all', False)

                # 检查是否已签到
                if not should_force and self.status_manager.is_signed_today(site_name):
                    status = self.status_manager.get_site_status(site_name)
                    logger.info(f"跳过已签到站点: {site_name} - {status.get('result', '已签到')} ({status.get('time', '')})")
                    skipped_entries.append({
                        'site': site_name,
                        'reason': f"已签到 - {status.get('result', '')}",
                        'time': status.get('time', ''),
                        'type': 'signed'
                    })
                    continue

                # 检查失败次数限制
                if not should_force:
                    should_skip, skip_reason = self.status_manager.should_skip_due_to_failures(
                        site_name, max_failed_attempts, retry_interval
                    )
                    if should_skip:
                        failed_count = self.status_manager.get_failed_count(site_name)
                        logger.warning(f"跳过失败过多站点: {site_name} - {skip_reason}")
                        skipped_entries.append({
                            'site': site_name,
                            'reason': skip_reason,
                            'time': '',
                            'type': 'failed_too_much',
                            'failed_count': failed_count
                        })
                        continue

                # 如果是强制签到，清除之前的状态（但保留失败次数）
                if should_force:
                    self.status_manager.clear_site_status(site_name, keep_failed_count=True)
                    if force_options.get('force_all'):
                        logger.info(f"强制重新签到所有站点，包括: {site_name}")

                valid_entries.append(entry)

            if not valid_entries:
                if skipped_entries:
                    logger.info(f"所有站点今日已签到，跳过 {len(skipped_entries)} 个站点")
                    # 显示跳过的站点摘要
                    for skipped in skipped_entries:
                        logger.info(f"  - {skipped['site']}: {skipped['reason']}")
                else:
                    logger.info("没有需要签到的条目")
                return
            
            logger.info(f"开始签到 {len(valid_entries)} 个站点")
            
            # 执行签到
            max_workers = self.config_manager.get_max_workers()
            success_count = 0
            failed_count = 0
            success_results = []
            failed_results = []

            with ThreadPoolExecutor(max_workers=max_workers) as executor:
                futures = []
                for entry in valid_entries:
                    future = executor.submit(self._sign_in_with_error_handling, entry, config)
                    futures.append((entry, future))

                # 等待所有任务完成
                for entry, future in futures:
                    try:
                        future.result()
                        if entry.failed:
                            failed_count += 1
                            failed_results.append({
                                'site': entry['site_name'],
                                'reason': entry.reason
                            })
                            # 记录签到失败状态
                            self.status_manager.record_signin_failed(entry['site_name'], entry.reason)
                            logger.error(f"签到失败: {entry['title']} - {entry.reason}")
                        else:
                            success_count += 1
                            success_results.append({
                                'site': entry['site_name'],
                                'result': entry.get('result', '签到成功'),
                                'messages': entry.get('messages', ''),
                                'details': entry.get('details', '')
                            })
                            # 记录签到成功状态
                            self.status_manager.record_signin_success(
                                entry['site_name'],
                                entry.get('result', '签到成功'),
                                entry.get('messages', ''),
                                entry.get('details', '')
                            )
                            logger.info(f"签到成功: {entry['title']} - {entry.get('result', '')}")
                    except Exception as e:
                        failed_count += 1
                        failed_results.append({
                            'site': entry['site_name'],
                            'reason': f"签到异常: {e}"
                        })
                        # 记录签到异常状态
                        self.status_manager.record_signin_failed(entry['site_name'], f"签到异常: {e}")
                        logger.exception(f"签到异常: {entry['title']} - {e}")
            
            # 统计结果
            end_time = datetime.now()
            duration = (end_time - start_time).total_seconds()

            # 构建详细的通知内容
            notification_lines = [
                f"📊 签到任务完成",
                f"总计: {len(valid_entries)} 个站点 | 成功: {success_count} 个 | 失败: {failed_count} 个",
                f"⏱️ 耗时: {duration:.2f} 秒",
                ""
            ]

            # 添加成功站点详情
            if success_results:
                notification_lines.append("✅ 签到成功:")
                for result in success_results:
                    site_info = f"  • {result['site']}: {result['result']}"
                    notification_lines.append(site_info)

                    # 如果有消息，添加消息信息
                    if result['messages']:
                        notification_lines.append(f"    📨 消息: {result['messages'][:100]}...")

                    # 如果有详情，添加详情信息
                    if result['details']:
                        details_str = str(result['details'])[:100]
                        notification_lines.append(f"    📈 详情: {details_str}...")

                notification_lines.append("")

            # 添加失败站点详情
            if failed_results:
                notification_lines.append("❌ 签到失败:")
                for result in failed_results:
                    notification_lines.append(f"  • {result['site']}: {result['reason'][:100]}...")
                notification_lines.append("")

            # 添加跳过的站点信息
            if skipped_entries:
                # 分类显示跳过的站点
                signed_skipped = [s for s in skipped_entries if s.get('type') == 'signed']
                failed_skipped = [s for s in skipped_entries if s.get('type') == 'failed_too_much']

                if signed_skipped:
                    notification_lines.append("⏭️ 跳过已签到:")
                    for skipped in signed_skipped:
                        notification_lines.append(f"  • {skipped['site']}: {skipped['reason']} ({skipped['time']})")
                    notification_lines.append("")

                if failed_skipped:
                    notification_lines.append("🚫 跳过失败过多:")
                    for skipped in failed_skipped:
                        failed_count = skipped.get('failed_count', 0)
                        notification_lines.append(f"  • {skipped['site']}: {skipped['reason']} (失败{failed_count}次)")
                    notification_lines.append("")

            # 添加时间信息
            notification_lines.append(f"🕐 执行时间: {start_time.strftime('%Y-%m-%d %H:%M:%S')}")

            result_message = "\n".join(notification_lines)

            logger.info(f"签到任务完成 - 成功: {success_count}, 失败: {failed_count}, 跳过: {len(skipped_entries)}, 耗时: {duration:.2f}秒")

            # 记录签到结果到日志
            logger.info(f"签到结果摘要:\n{result_message}")

            # 清理旧记录
            self.status_manager.cleanup_old_records()
            
        except Exception as e:
            error_msg = f"签到任务执行异常: {e}"
            logger.exception(error_msg)
            # 记录错误到日志
            logger.error(f"签到任务执行失败: {error_msg}")
    
    def _sign_in_with_error_handling(self, entry: SignInEntry, config: dict) -> None:
        """带错误处理的签到"""
        try:
            from .executor import sign_in
            sign_in(entry, config)
        except Exception as e:
            entry.fail(f"签到异常: {e}")
            logger.exception(f"站点 {entry['site_name']} 签到异常: {e}")
    
    def run_once(self, force_options: dict = None) -> None:
        """立即执行一次签到任务"""
        logger.info("立即执行签到任务")
        self.run_sign_in_task(force_options)
    
    def start(self) -> None:
        """启动调度器"""
        self.running = True
        self.setup_schedule()
        logger.info("任务调度器已启动")
        
        try:
            while self.running:
                schedule.run_pending()
                time.sleep(60)  # 每分钟检查一次
        except KeyboardInterrupt:
            logger.info("收到中断信号，正在停止...")
        finally:
            self.stop()
    
    def stop(self) -> None:
        """停止调度器"""
        self.running = False
        schedule.clear()
        logger.info("任务调度器已停止")
